@@ -7,6 +7,8 @@
 //
 
 #import "PSMTabBarControl.h"
+
+#import "DebugLogging.h"
 #import "PSMTabBarCell.h"
 #import "PSMOverflowPopUpButton.h"
 #import "PSMRolloverButton.h"
@@ -20,9 +22,8 @@ NSString *const kPSMModifierChangedNotification = @"kPSMModifierChangedNotificat
 NSString *const kPSMTabModifierKey = @"TabModifier";
 NSString *const PSMTabDragDidEndNotification = @"PSMTabDragDidEndNotification";
 NSString *const PSMTabDragDidBeginNotification = @"PSMTabDragDidBeginNotification";
-const CGFloat kPSMTabBarControlHeight = 22;
+const CGFloat kPSMTabBarControlHeight = 24;
 const CGFloat kSPMTabBarCellInternalXMargin = 6;
-const CGFloat kSPMTabBarCellInternalYMargin = 3.5;
 
 const CGFloat kPSMTabBarCellPadding = 4;
 const CGFloat kPSMTabBarCellIconPadding = 0;
@@ -66,6 +67,8 @@ const NSInteger kPSMStartResizeAnimation = 0;
     // drag and drop
     NSEvent *_lastMouseDownEvent; // keep this for dragging reference
     NSEvent *_lastMiddleMouseDownEvent;
+    BOOL _haveInitialDragLocation;
+    NSPoint _initialDragLocation;
     BOOL _didDrag;
     BOOL _closeClicked;
 
@@ -101,8 +104,7 @@ const NSInteger kPSMStartResizeAnimation = 0;
 #pragma mark -
 #pragma mark Constructor/destructor
 
-- (id)initWithFrame:(NSRect)frame
-{
+- (id)initWithFrame:(NSRect)frame {
     self = [super initWithFrame:frame];
     if (self) {
         // Initialization
@@ -118,6 +120,7 @@ const NSInteger kPSMStartResizeAnimation = 0;
         _cellMinWidth = 100;
         _cellMaxWidth = 280;
         _cellOptimumWidth = 130;
+        _minimumTabDragDistance = 10;
         _hasCloseButton = YES;
         _tabLocation = PSMTab_TopTab;
         _style = [[PSMYosemiteTabStyle alloc] init];
@@ -125,10 +128,10 @@ const NSInteger kPSMStartResizeAnimation = 0;
         // the overflow button/menu
         NSRect overflowButtonRect = NSMakeRect([self frame].size.width - [_style rightMarginForTabBarControl] + 1, 0, [_style rightMarginForTabBarControl] - 1, [self frame].size.height);
         _overflowPopUpButton = [[PSMOverflowPopUpButton alloc] initWithFrame:overflowButtonRect pullsDown:YES];
-        if (_overflowPopUpButton){
+        if (_overflowPopUpButton) {
             // configure
             [_overflowPopUpButton setAutoresizingMask:NSViewNotSizable|NSViewMinXMargin];
-            [[_overflowPopUpButton cell] accessibilitySetOverrideValue:NSLocalizedStringFromTableInBundle(@"More tabs", @"iTerm", [NSBundle bundleForClass:[self class]], @"VoiceOver label for the button displaying menu of additional overflown tabs on click") forAttribute:NSAccessibilityDescriptionAttribute];
+            _overflowPopUpButton.accessibilityLabel = @"More tabs";
         }
 
         // new tab button
@@ -215,6 +218,37 @@ const NSInteger kPSMStartResizeAnimation = 0;
     }
 }
 
+- (void)sanityCheckFailedWithCallsite:(NSString *)callsite reason:(NSString *)reason {
+    ILog(@"Sanity check failed from %@ for reason %@. Cells=%@. tabView.tabViewItems=%@ stack:\n%@",
+         callsite,
+         reason,
+         self.cells,
+         self.tabView.tabViewItems,
+         [NSThread callStackSymbols]);
+}
+
+- (void)sanityCheck:(NSString *)callsite {
+    [self sanityCheck:callsite force:NO];
+}
+
+- (void)sanityCheck:(NSString *)callsite force:(BOOL)force {
+    if (!force && [[PSMTabDragAssistant sharedDragAssistant] isDragging]) {
+        ILog(@"Skip sanity check during drag from callsite %@", callsite);
+        return;
+    }
+    if (self.tabView.tabViewItems.count != self.cells.count) {
+        [self sanityCheckFailedWithCallsite:callsite reason:@"count mismatch"];
+    } else {
+        for (NSInteger i = 0; i < self.cells.count; i++) {
+            NSTabViewItem *tabViewItem = self.tabView.tabViewItems[i];
+            PSMTabBarCell *cell = self.cells[i];
+            if (cell.representedObject != tabViewItem) {
+                [self sanityCheckFailedWithCallsite:callsite reason:@"cells[i].representedObject != tabView.tabViewItems[i].representedObject"];
+            }
+        }
+        NSLog(@"Sanity check passed. cells=%@. tabView.tabViewITems=%@", self.cells, self.tabView.tabViewItems);
+    }
+}
 
 #pragma mark -
 #pragma mark Accessors
@@ -336,6 +370,11 @@ const NSInteger kPSMStartResizeAnimation = 0;
     [self update:_automaticallyAnimates];
 }
 
+- (void)setStretchCellsToFit:(BOOL)value {
+    _stretchCellsToFit = value;
+    [self update:_automaticallyAnimates];
+}
+
 - (void)setUseOverflowMenu:(BOOL)value {
     _useOverflowMenu = value;
     [self update];
@@ -719,22 +758,21 @@ const NSInteger kPSMStartResizeAnimation = 0;
     [self update:YES];
 }
 
-- (void)update
-{
+- (void)update {
     [self update:NO];
 }
 
-- (void)update:(BOOL)animate
-{
-    // abandon hope, all ye who enter here :-)
-    // this method handles all of the cell layout, and is called when something changes to require the refresh.  This method is not called during drag and drop; see the PSMTabDragAssistant's calculateDragAnimationForTabBar: method, which does layout in that case.
+- (void)update:(BOOL)animate {
+    // This method handles all of the cell layout, and is called when something changes to require
+    // the refresh.  This method is not called during drag and drop. See the PSMTabDragAssistant's
+    // calculateDragAnimationForTabBar: method, which does layout in that case.
 
-    // make sure all of our tabs are accounted for before updating
+    // Make sure all of our tabs are accounted for before updating.
     if ([_tabView numberOfTabViewItems] != [_cells count]) {
         return;
     }
 
-    // hide/show? (these return if already in desired state)
+    // Hide or show? These do nothing if already in the desired state.
     if ((_hideForSingleTab) && ([_cells count] <= 1)) {
         [self hideTabBar:YES animate:YES];
     } else {
@@ -743,198 +781,142 @@ const NSInteger kPSMStartResizeAnimation = 0;
 
     [self _removeCellTrackingRects];
 
-    // calculate number of cells to fit in control and cell widths
-    int i, cellCount = [_cells count];
-    float availableWidth = [self availableCellWidth], currentOrigin = 0;
-    NSMutableArray *newWidths = [NSMutableArray arrayWithCapacity:cellCount];
-    int numberOfVisibleCells = ([self orientation] == PSMTabBarHorizontalOrientation) ? 1 : 0;
-    float totalOccupiedWidth = 0.0;
-    NSRect cellRect = [self genericCellRect];
-
-    if ([self orientation] == PSMTabBarVerticalOrientation) {
-        currentOrigin = [[self style] topMarginForTabBarControl];
-    }
-
     NSLineBreakMode truncationStyle = [self truncationStyle];
-    
-    for (i = 0; i < cellCount; i++) {
-        PSMTabBarCell *cell = [_cells objectAtIndex:i];
+
+    // Update cells' settings in case they changed.
+    for (PSMTabBarCell *cell in _cells) {
         cell.truncationStyle = truncationStyle;
         cell.hasCloseButton = _hasCloseButton;
         [cell updateForStyle];
-        float width;
-
-        // suppress close button?
         cell.isCloseButtonSuppressed = [self disableTabClose];
-
-        if ([self orientation] == PSMTabBarHorizontalOrientation) {
-            // Determine cell width
-            if (_sizeCellsToFit) {
-                width = MAX(_cellMinWidth, [cell desiredWidthOfCell]);
-                if (width > _cellMaxWidth) {
-                    width = _cellMaxWidth;
-                }
-            } else {
-                width = _cellOptimumWidth;
-            }
-
-            //check to see if there is not enough space to place all tabs as preferred
-            totalOccupiedWidth += width;
-            if (totalOccupiedWidth >= availableWidth) {
-                //if we're not going to use the overflow menu, cram all the tab cells into the bar
-                if (!_useOverflowMenu) {
-                    int j, averageWidth = (availableWidth / cellCount);
-
-                    [newWidths removeAllObjects];
-
-                    for (j = 0; j < cellCount; j++) {
-                        float desiredWidth = [[_cells objectAtIndex:j] desiredWidthOfCell];
-                        [newWidths addObject:[NSNumber numberWithFloat:(desiredWidth < averageWidth && [self sizeCellsToFit]) ? desiredWidth : averageWidth]];
-                    }
-                    break;
-                }
-
-                numberOfVisibleCells = i;
-                if (_sizeCellsToFit) {
-                    int neededWidth = width - (totalOccupiedWidth - availableWidth); //the amount of space needed to fit the next cell in
-                    // can I squeeze it in without violating min cell width?
-                    int widthIfAllMin = (numberOfVisibleCells + 1) * _cellMinWidth;
-
-                    if ((width + widthIfAllMin) <= availableWidth) {
-                        // squeeze - distribute needed sacrifice among all cells
-                        int q;
-                        for (q = (i - 1); q >= 0; q--) {
-                            int desiredReduction = (int)neededWidth / (q + 1);
-                            if (([[newWidths objectAtIndex:q] floatValue] - desiredReduction) < _cellMinWidth) {
-                                int actualReduction = (int)[[newWidths objectAtIndex:q] floatValue] - _cellMinWidth;
-                                [newWidths replaceObjectAtIndex:q withObject:[NSNumber numberWithFloat:_cellMinWidth]];
-                                neededWidth -= actualReduction;
-                            } else {
-                                int newCellWidth = (int)[[newWidths objectAtIndex:q] floatValue] - desiredReduction;
-                                [newWidths replaceObjectAtIndex:q withObject:[NSNumber numberWithFloat:newCellWidth]];
-                                neededWidth -= desiredReduction;
-                            }
-                        }
-
-                        int totalWidth = [[newWidths valueForKeyPath:@"@sum.intValue"] intValue];
-                        int thisWidth = width - neededWidth; //width the last cell would want
-
-                        //append a final cell if there is enough room, otherwise stretch all the cells out to fully fit the bar
-                        if (availableWidth - totalWidth > thisWidth) {
-                            [newWidths addObject:[NSNumber numberWithFloat:thisWidth]];
-                            numberOfVisibleCells++;
-                            totalWidth += thisWidth;
-                        }
-
-                        if (totalWidth < availableWidth) {
-                            int leftoverWidth = availableWidth - totalWidth;
-                            int q2;
-
-                            for (q2 = i - 1; q2 >= 0; q2--) {
-                                int desiredAddition = (int)leftoverWidth / (q2 + 1);
-                                int newCellWidth = (int)[[newWidths objectAtIndex:q2] floatValue] + desiredAddition;
-                                [newWidths replaceObjectAtIndex:q2 withObject:[NSNumber numberWithFloat:newCellWidth]];
-                                leftoverWidth -= desiredAddition;
-                            }
-                        }
-                    } else {
-                        // stretch - distribute leftover room among cells
-                        int leftoverWidth = availableWidth - totalOccupiedWidth + width;
-                        int q;
-
-                        for (q = i - 1; q >= 0; q--) {
-                            int desiredAddition = (int)leftoverWidth / (q + 1);
-                            int newCellWidth = (int)[[newWidths objectAtIndex:q] floatValue] + desiredAddition;
-                            [newWidths replaceObjectAtIndex:q withObject:[NSNumber numberWithFloat:newCellWidth]];
-                            leftoverWidth -= desiredAddition;
-                        }
-                    }
-
-                    //make sure there are at least two items in the tab bar
-                    if (numberOfVisibleCells < 2 && [_cells count] > 1) {
-                        PSMTabBarCell *cell1 = [_cells objectAtIndex:0], *cell2 = [_cells objectAtIndex:1];
-                        NSNumber *cellWidth;
-
-                        [newWidths removeAllObjects];
-                        totalOccupiedWidth = 0;
-
-                        cellWidth = [NSNumber numberWithFloat:[cell1 desiredWidthOfCell] < availableWidth * 0.5f ? [cell1 desiredWidthOfCell] : availableWidth * 0.5f];
-                        [newWidths addObject:cellWidth];
-                        totalOccupiedWidth += [cellWidth floatValue];
-
-                        cellWidth = [NSNumber numberWithFloat:[cell2 desiredWidthOfCell] < (availableWidth - totalOccupiedWidth) ? [cell2 desiredWidthOfCell] : (availableWidth - totalOccupiedWidth)];
-                        [newWidths addObject:cellWidth];
-                        totalOccupiedWidth += [cellWidth floatValue];
-
-                        if (totalOccupiedWidth < availableWidth) {
-                            [newWidths replaceObjectAtIndex:0 withObject:[NSNumber numberWithFloat:availableWidth - [cellWidth floatValue]]];
-                        }
-                    }
-
-                    break; // done assigning widths; remaining cells go in overflow menu
-                } else {
-                    int revisedWidth = availableWidth / (i + 1);
-                    if (revisedWidth >= _cellMinWidth) {
-                        int q;
-                        totalOccupiedWidth = 0;
-                        for (q = 0; q < [newWidths count]; q++) {
-                            [newWidths replaceObjectAtIndex:q withObject:[NSNumber numberWithFloat:revisedWidth]];
-                            totalOccupiedWidth += revisedWidth;
-                        }
-                        // just squeezed this one in...
-                        [newWidths addObject:[NSNumber numberWithFloat:revisedWidth]];
-                        totalOccupiedWidth += revisedWidth;
-                        numberOfVisibleCells++;
-                    } else {
-                        // couldn't fit that last one...
-                        break;
-                    }
-                }
-            } else {
-                numberOfVisibleCells = cellCount;
-                [newWidths addObject:[NSNumber numberWithFloat:width]];
-            }
-        } else {
-            //lay out vertical tabs
-            if (currentOrigin + cellRect.size.height <= [self frame].size.height) {
-                [newWidths addObject:[NSNumber numberWithFloat:currentOrigin]];
-                numberOfVisibleCells++;
-                currentOrigin += cellRect.size.height;
-            } else {
-                //out of room, the remaining tabs go into overflow
-                if ([newWidths count] > 0 && [self frame].size.height - currentOrigin < 17) {
-                    [newWidths removeLastObject];
-                    numberOfVisibleCells--;
-                }
-                break;
-            }
-        }
-    }
-
-    for (PSMTabBarCell *cell in _cells) {
         // Remove highlight if cursor is no longer in cell. Could happen if
         // cell moves because of added/removed tab. Tracking rects aren't smart
         // enough to handle this.
         [cell updateHighlight];
     }
 
-    if ((animate || _animationTimer != nil) && [self orientation] == PSMTabBarHorizontalOrientation && [_cells count] > 0) {
-        //animate only on horizontal tab bars
-        if (_animationTimer) {
-            [_animationTimer invalidate];
+    // Calculate number of cells to fit in the control and cell widths.
+    const NSInteger cellCount = [_cells count];
+    if ([self orientation] == PSMTabBarHorizontalOrientation) {
+        if ((animate || _animationTimer != nil) && cellCount > 0) {
+            // Animate only on horizontal tab bars.
+            if (_animationTimer) {
+                [_animationTimer invalidate];
+            }
+            
+            _animationDelta = 0.0f;
+            _animationTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 / 30.0
+                                                               target:self
+                                                             selector:@selector(_animateCells:)
+                                                             userInfo:[self cellWidthsForHorizontalArrangement]
+                                                              repeats:YES];
+            return;
+        } else {
+            [self finishUpdate:[self cellWidthsForHorizontalArrangement]];
         }
-
-        _animationDelta = 0.0f;
-        _animationTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 / 30.0 target:self selector:@selector(_animateCells:) userInfo:newWidths repeats:YES];
     } else {
-        [self _finishCellUpdate:newWidths];
-        [self setNeedsDisplay];
+        // Vertical orientation
+        CGFloat currentOrigin = [[self style] topMarginForTabBarControl];
+        NSRect cellRect = [self genericCellRect];
+        NSMutableArray *newOrigins = [NSMutableArray arrayWithCapacity:cellCount];
+
+        for (int i = 0; i < cellCount; ++i) {
+            // Lay out vertical tabs.
+            if (currentOrigin + cellRect.size.height <= [self frame].size.height) {
+                [newOrigins addObject:[NSNumber numberWithFloat:currentOrigin]];
+                currentOrigin += cellRect.size.height;
+            } else {
+                // Out of room, the remaining tabs go into overflow.
+                if ([newOrigins count] > 0 && [self frame].size.height - currentOrigin < 17) {
+                    [newOrigins removeLastObject];
+                }
+                break;
+            }
+        }
+        [self finishUpdate:newOrigins];
     }
 
+    [self setNeedsDisplay];
 }
 
-- (void)_removeCellTrackingRects
-{
+// Tab widths may vary. Calculate the widths and see if this will work. Only allow sizes to
+// vary if all tabs fit in the allotted space.
+- (NSArray<NSNumber *> *)variableCellWidths {
+    const CGFloat availableWidth = [self availableCellWidth];
+    CGFloat totalDesiredWidth = 0.0;
+    NSMutableArray *desiredWidths = [NSMutableArray array];
+    for (PSMTabBarCell *cell in _cells) {
+        const CGFloat width = MAX(_cellMinWidth, MIN([cell desiredWidthOfCell], _cellMaxWidth));
+        [desiredWidths addObject:@(width)];
+        totalDesiredWidth += width;
+        if (totalDesiredWidth > availableWidth) {
+            // Stop this expensive calculation as soon as it fails.
+            break;
+        }
+    }
+    
+    // If all cells get their "desired" width, do they fit?
+    if (totalDesiredWidth <= availableWidth) {
+        return desiredWidths;
+    } else {
+        return nil;
+    }
+}
+
+- (BOOL)shouldUseOptimalWidth {
+    const CGFloat availableWidth = [self availableCellWidth];
+    // If all cells are the client-specified optimum size, do they fit?
+    BOOL canFitAllCellsOptimally = (self.cellOptimumWidth * _cells.count <= availableWidth);
+    return !self.stretchCellsToFit && canFitAllCellsOptimally;
+}
+
+- (NSArray<NSNumber *> *)cellWidthsForHorizontalArrangement {
+    const NSUInteger cellCount = _cells.count;
+    const CGFloat availableWidth = [self availableCellWidth];
+    
+    if (self.sizeCellsToFit) {
+        NSArray<NSNumber *> *widths = [self variableCellWidths];
+        if (widths) {
+            return widths;
+        }
+    }
+    
+    NSMutableArray<NSNumber *> *newWidths = [NSMutableArray array];
+    if ([self shouldUseOptimalWidth]) {
+        // Use the client-specified size, even if that leaves unused space on the right.
+        for (int i = 0; i < cellCount; i++) {
+            [newWidths addObject:@(_cellOptimumWidth)];
+        }
+    } else {
+        // Divide up the space evenly, but don't allow cells to be smaller than the minimum
+        // width.
+        CGFloat widthRemaining = availableWidth;
+        // If all cells are the smallest allowed size, do they fit?
+        const BOOL canFitAllCellsMinimally = (self.cellMinWidth * cellCount <= availableWidth);
+        NSInteger numberOfVisibleCells = canFitAllCellsMinimally ? cellCount : (availableWidth / _cellMinWidth);
+        for (int i = 0; i < numberOfVisibleCells; i++) {
+            CGFloat width = floor(widthRemaining / (numberOfVisibleCells - i));
+            [newWidths addObject:@(width)];
+            widthRemaining -= width;
+        }
+    }
+    return newWidths;
+}
+
+- (void)removeCell:(PSMTabBarCell *)cell {
+    if ([cell closeButtonTrackingTag] != 0) {
+        [self removeTrackingRect:[cell closeButtonTrackingTag]];
+        [cell setCloseButtonTrackingTag:0];
+    }
+
+    if ([cell cellTrackingTag] != 0) {
+        [self removeTrackingRect:[cell cellTrackingTag]];
+        [cell setCellTrackingTag:0];
+    }
+    [[self cells] removeObject:cell];
+}
+
+- (void)_removeCellTrackingRects {
     // size all cells appropriately and create tracking rects
     // nuke old tracking rects
     int i, cellCount = [_cells count];
@@ -956,8 +938,7 @@ const NSInteger kPSMStartResizeAnimation = 0;
     [self removeAllToolTips];
 }
 
-- (void)_animateCells:(NSTimer *)timer
-{
+- (void)_animateCells:(NSTimer *)timer {
     NSArray *targetWidths = [timer userInfo];
     int i, numberOfVisibleCells = [targetWidths count];
     float totalChange = 0.0f;
@@ -996,7 +977,7 @@ const NSInteger kPSMStartResizeAnimation = 0;
     }
 
     if (!updated) {
-        [self _finishCellUpdate:targetWidths];
+        [self finishUpdate:targetWidths];
         [timer invalidate];
         _animationTimer = nil;
     }
@@ -1004,10 +985,9 @@ const NSInteger kPSMStartResizeAnimation = 0;
     [self setNeedsDisplay];
 }
 
-- (void)_finishCellUpdate:(NSArray *)newWidths
-{
-    //setup overflow menu
-    NSMenu *overflowMenu = [self _setupCells:newWidths];
+- (void)finishUpdate:(NSArray *)newValues {
+    // Set up overflow menu.
+    NSMenu *overflowMenu = [self _setupCells:newValues];
 
     if (overflowMenu) {
         [self _setupOverflowMenu:overflowMenu];
@@ -1015,17 +995,17 @@ const NSInteger kPSMStartResizeAnimation = 0;
 
     [_overflowPopUpButton setHidden:(overflowMenu == nil)];
 
-    //setup add tab button
+    // Set up add tab button.
     if (!overflowMenu && _showAddTabButton) {
         NSRect cellRect = [self genericCellRect];
         cellRect.size = [_addTabButton frame].size;
 
         if ([self orientation] == PSMTabBarHorizontalOrientation) {
-            cellRect.origin.y = kSPMTabBarCellInternalYMargin;
-            cellRect.origin.x += [[newWidths valueForKeyPath:@"@sum.floatValue"] floatValue] + 2;
+            cellRect.origin.y = 0;
+            cellRect.origin.x += [[newValues valueForKeyPath:@"@sum.floatValue"] floatValue] + 2;
         } else {
             cellRect.origin.x = 0;
-            cellRect.origin.y = [[newWidths lastObject] floatValue];
+            cellRect.origin.y = [[newValues lastObject] floatValue];
         }
 
         [self _setupAddTabButton:cellRect];
@@ -1034,9 +1014,9 @@ const NSInteger kPSMStartResizeAnimation = 0;
     }
 }
 
-- (NSMenu *)_setupCells:(NSArray *)newWidths {
+- (NSMenu *)_setupCells:(NSArray *)newValues {
     NSRect cellRect = [self genericCellRect];
-    int i, cellCount = [_cells count], numberOfVisibleCells = [newWidths count];
+    int i, cellCount = [_cells count], numberOfVisibleCells = [newValues count];
     NSMenu *overflowMenu = nil;
 
     // Set up cells with frames and rects
@@ -1046,10 +1026,10 @@ const NSInteger kPSMStartResizeAnimation = 0;
         if (i < numberOfVisibleCells) {
             // set cell frame
             if ([self orientation] == PSMTabBarHorizontalOrientation) {
-                cellRect.size.width = [[newWidths objectAtIndex:i] floatValue];
+                cellRect.size.width = [[newValues objectAtIndex:i] floatValue];
             } else {
                 cellRect.size.width = [self frame].size.width;
-                cellRect.origin.y = [[newWidths objectAtIndex:i] floatValue];
+                cellRect.origin.y = [[newValues objectAtIndex:i] floatValue];
                 cellRect.origin.x = 0;
             }
             [cell setFrame:cellRect];
@@ -1134,7 +1114,7 @@ const NSInteger kPSMStartResizeAnimation = 0;
             }
 
             // next...
-            cellRect.origin.x += [[newWidths objectAtIndex:i] floatValue];
+            cellRect.origin.x += [[newValues objectAtIndex:i] floatValue];
 
         } else {
             // set up menu items
@@ -1294,6 +1274,11 @@ const NSInteger kPSMStartResizeAnimation = 0;
     if ([self lastMouseDownEvent] == nil) {
         return;
     }
+    if (!_haveInitialDragLocation) {
+        _initialDragLocation = [theEvent locationInWindow];
+        _haveInitialDragLocation = YES;
+        return;
+    }
 
     NSPoint currentPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
 
@@ -1320,7 +1305,7 @@ const NSInteger kPSMStartResizeAnimation = 0;
     }
 
     NSRect cellFrame;
-    NSPoint trackingStartPoint = [self convertPoint:[[self lastMouseDownEvent] locationInWindow] fromView:nil];
+    NSPoint trackingStartPoint = [self convertPoint:_initialDragLocation fromView:nil];
     PSMTabBarCell *cell = [self cellForPoint:trackingStartPoint cellFrame:&cellFrame];
     if (cell) {
         //check to see if the close button was the target in the clicked cell
@@ -1338,10 +1323,11 @@ const NSInteger kPSMStartResizeAnimation = 0;
         float dy = fabs(currentPoint.y - trackingStartPoint.y);
         float distance = sqrt(dx * dx + dy * dy);
 
-        if (distance >= 10 && !_didDrag && ![[PSMTabDragAssistant sharedDragAssistant] isDragging] &&
+        if (distance >= self.minimumTabDragDistance && !_didDrag && ![[PSMTabDragAssistant sharedDragAssistant] isDragging] &&
                 [[self delegate] respondsToSelector:@selector(tabView:shouldDragTabViewItem:fromTabBar:)] &&
                 [[self delegate] tabView:_tabView shouldDragTabViewItem:[cell representedObject] fromTabBar:self]) {
             _didDrag = YES;
+            ILog(@"Start dragging with mouse down event %@ in window %p with frame %@", [self lastMouseDownEvent], self.window, NSStringFromRect(self.window.frame));
             [[PSMTabDragAssistant sharedDragAssistant] startDraggingCell:cell fromTabBar:self withMouseDownEvent:[self lastMouseDownEvent]];
         }
     }
@@ -1365,6 +1351,7 @@ const NSInteger kPSMStartResizeAnimation = 0;
 
 - (void)mouseUp:(NSEvent *)theEvent
 {
+    _haveInitialDragLocation = NO;
     if (_resizing) {
         _resizing = NO;
         [[NSCursor arrowCursor] set];
@@ -1437,7 +1424,14 @@ const NSInteger kPSMStartResizeAnimation = 0;
     return YES;
 }
 
-// NSDraggingSource
+#pragma mark NSDraggingSource
+
+- (NSDraggingSession *)beginDraggingSessionWithItems:(NSArray<NSDraggingItem *> *)items event:(NSEvent *)event source:(id<NSDraggingSource>)source {
+    ILog(@"Begin dragging tab bar control %p with event %@ source from\n%@",
+         self, event, [NSThread callStackSymbols]);
+    return [super beginDraggingSessionWithItems:items event:event source:source];
+}
+
 - (NSDragOperation)draggingSourceOperationMaskForLocal:(BOOL)isLocal
 {
     return (isLocal ? NSDragOperationMove : NSDragOperationNone);
@@ -1448,19 +1442,17 @@ const NSInteger kPSMStartResizeAnimation = 0;
     return YES;
 }
 
-- (void)draggedImage:(NSImage *)anImage beganAt:(NSPoint)screenPoint
-{
+- (void)draggingSession:(NSDraggingSession *)session willBeginAtPoint:(NSPoint)screenPoint {
     [[PSMTabDragAssistant sharedDragAssistant] draggingBeganAt:screenPoint];
 }
 
-- (void)draggedImage:(NSImage *)image movedTo:(NSPoint)screenPoint
-{
+- (void)draggingSession:(NSDraggingSession *)session movedToPoint:(NSPoint)screenPoint {
     [[PSMTabDragAssistant sharedDragAssistant] draggingMovedTo:screenPoint];
 }
 
-// NSDraggingDestination
-- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
-{
+#pragma mark NSDraggingDestination
+
+- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender {
     if ([[[sender draggingPasteboard] types] indexOfObject:@"com.iterm2.psm.controlitem"] != NSNotFound) {
         if ([[self delegate] respondsToSelector:@selector(tabView:shouldDropTabViewItem:inTabBar:)] &&
             ![[self delegate] tabView:[[sender draggingSource] tabView]
@@ -1483,8 +1475,7 @@ const NSInteger kPSMStartResizeAnimation = 0;
     }
 }
 
-- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
-{
+- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender {
     PSMTabBarCell *cell = [self cellForPoint:[self convertPoint:[sender draggingLocation] fromView:nil] cellFrame:nil];
 
     if ([[[sender draggingPasteboard] types] indexOfObject:@"com.iterm2.psm.controlitem"] != NSNotFound) {
@@ -1511,13 +1502,11 @@ const NSInteger kPSMStartResizeAnimation = 0;
     return NSDragOperationNone;
 }
 
-- (void)draggingExited:(id <NSDraggingInfo>)sender
-{
+- (void)draggingExited:(id <NSDraggingInfo>)sender {
     [[PSMTabDragAssistant sharedDragAssistant] draggingExitedTabBar:self];
 }
 
-- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
-{
+- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender {
     // validate the drag operation only if there's a valid tab bar to drop into
     BOOL badType = [[[sender draggingPasteboard] types] indexOfObject:@"com.iterm2.psm.controlitem"] == NSNotFound;
     if (badType && [[self delegate] respondsToSelector:@selector(tabView:shouldAcceptDragFromSender:)] &&
@@ -1528,14 +1517,13 @@ const NSInteger kPSMStartResizeAnimation = 0;
            [[PSMTabDragAssistant sharedDragAssistant] destinationTabBar] != nil;
 }
 
-- (BOOL)_delegateAcceptsSender:(id <NSDraggingInfo>)sender
-{
+- (BOOL)_delegateAcceptsSender:(id <NSDraggingInfo>)sender {
     return [[self delegate] respondsToSelector:@selector(tabView:shouldAcceptDragFromSender:)] &&
            [[self delegate] tabView:_tabView shouldAcceptDragFromSender:sender];
 }
 
-- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
-{
+- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender {
+    _haveInitialDragLocation = NO;
     if ([[[sender draggingPasteboard] types] indexOfObject:@"com.iterm2.psm.controlitem"] != NSNotFound ||
         [self _delegateAcceptsSender:sender]) {
         [[PSMTabDragAssistant sharedDragAssistant] performDragOperation:sender];
@@ -1546,8 +1534,8 @@ const NSInteger kPSMStartResizeAnimation = 0;
     return YES;
 }
 
-- (void)draggedImage:(NSImage *)anImage endedAt:(NSPoint)aPoint operation:(NSDragOperation)operation
-{
+- (void)draggingSession:(NSDraggingSession *)session endedAtPoint:(NSPoint)aPoint operation:(NSDragOperation)operation {
+    _haveInitialDragLocation = NO;
     if (operation != NSDragOperationNone) {
         [self removeTabForCell:[[PSMTabDragAssistant sharedDragAssistant] draggedCell]];
         [[PSMTabDragAssistant sharedDragAssistant] finishDrag];
@@ -1556,11 +1544,6 @@ const NSInteger kPSMStartResizeAnimation = 0;
     }
 }
 
-- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
-{
-
-}
-
 #pragma mark -
 #pragma mark Actions
 
@@ -1583,8 +1566,8 @@ const NSInteger kPSMStartResizeAnimation = 0;
         }
     }
 
-    if ([[self delegate] respondsToSelector:@selector(closeTab:)]) {
-        [[self delegate] closeTab:[item identifier]];
+    if ([[self delegate] respondsToSelector:@selector(tabView:closeTab:)]) {
+        [[self delegate] tabView:[self tabView] closeTab:[item identifier]];
     }
 }
 
@@ -1850,6 +1833,7 @@ const NSInteger kPSMStartResizeAnimation = 0;
         [aCoder encodeBool:_selectsTabsOnMouseDown forKey:@"PSMselectsTabsOnMouseDown"];
         [aCoder encodeBool:_showAddTabButton forKey:@"PSMshowAddTabButton"];
         [aCoder encodeBool:_sizeCellsToFit forKey:@"PSMsizeCellsToFit"];
+        [aCoder encodeBool:_stretchCellsToFit forKey:@"PSMstretchCellsToFit"];
         [aCoder encodeInt:_cellMinWidth forKey:@"PSMcellMinWidth"];
         [aCoder encodeInt:_cellMaxWidth forKey:@"PSMcellMaxWidth"];
         [aCoder encodeInt:_cellOptimumWidth forKey:@"PSMcellOptimumWidth"];
@@ -1885,6 +1869,7 @@ const NSInteger kPSMStartResizeAnimation = 0;
             _selectsTabsOnMouseDown = [aDecoder decodeBoolForKey:@"PSMselectsTabsOnMouseDown"];
             _showAddTabButton = [aDecoder decodeBoolForKey:@"PSMshowAddTabButton"];
             _sizeCellsToFit = [aDecoder decodeBoolForKey:@"PSMsizeCellsToFit"];
+            _stretchCellsToFit = [aDecoder decodeBoolForKey:@"PSMstretchCellsToFit"];
             _cellMinWidth = [aDecoder decodeIntForKey:@"PSMcellMinWidth"];
             _cellMaxWidth = [aDecoder decodeIntForKey:@"PSMcellMaxWidth"];
             _cellOptimumWidth = [aDecoder decodeIntForKey:@"PSMcellOptimumWidth"];
@@ -2026,66 +2011,39 @@ const NSInteger kPSMStartResizeAnimation = 0;
 #pragma mark -
 #pragma mark Accessibility
 
--(BOOL)accessibilityIsIgnored {
-    return NO;
+- (NSString*)accessibilityRole {
+    return NSAccessibilityTabGroupRole;
 }
 
-- (NSArray*)accessibilityAttributeNames
-{
-    static NSArray *attributes = nil;
-    if (!attributes) {
-        NSSet *set = [NSSet setWithArray:[super accessibilityAttributeNames]];
-        set = [set setByAddingObjectsFromArray:[NSArray arrayWithObjects:
-                                                NSAccessibilityTabsAttribute,
-                                                NSAccessibilityValueAttribute,
-                                                nil]];
-        attributes = [[set allObjects] retain];
+- (NSArray*)accessibilityChildren {
+    NSMutableArray *childElements = [NSMutableArray array];
+    for (PSMTabBarCell *cell in [_cells objectsAtIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, [self numberOfVisibleTabs])]]) {
+        [childElements addObject:cell.element];
     }
-    return attributes;
+    if (![_overflowPopUpButton isHidden]) {
+        [childElements addObject:_overflowPopUpButton];
+    }
+    if (![_addTabButton isHidden]) {
+        [childElements addObject:_addTabButton];
+    }
+    return childElements;
 }
 
-- (id)accessibilityAttributeValue:(NSString *)attribute {
-    id attributeValue = nil;
-    if ([attribute isEqualToString: NSAccessibilityRoleAttribute]) {
-        attributeValue = NSAccessibilityTabGroupRole;
-    } else if ([attribute isEqualToString: NSAccessibilityChildrenAttribute]) {
-        NSMutableArray *children = [NSMutableArray arrayWithArray:[_cells objectsAtIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, [self numberOfVisibleTabs])]]];
-        if (![_overflowPopUpButton isHidden]) {
-            [children addObject:_overflowPopUpButton];
-        }
-        if (![_addTabButton isHidden]) {
-            [children addObject:_addTabButton];
-        }
-        attributeValue = NSAccessibilityUnignoredChildren(children);
-    } else if ([attribute isEqualToString: NSAccessibilityTabsAttribute]) {
-        attributeValue = NSAccessibilityUnignoredChildren(_cells);
-    } else if ([attribute isEqualToString:NSAccessibilityValueAttribute]) {
-        NSTabViewItem *tabViewItem = [_tabView selectedTabViewItem];
-        for (NSActionCell *cell in _cells) {
-            if ([cell representedObject] == tabViewItem)
-                attributeValue = cell;
-        }
-        if (!attributeValue)
-        {
-            NSLog(@"WARNING: seems no tab cell is currently selected");
-        }
-    } else {
-        attributeValue = [super accessibilityAttributeValue:attribute];
+- (NSArray*)accessibilityTabs {
+    NSMutableArray *tabElements = [NSMutableArray array];
+    for (PSMTabBarCell *cell in _cells) {
+        [tabElements addObject:cell.element];
     }
-    return attributeValue;
+    return tabElements;
 }
 
 - (id)accessibilityHitTest:(NSPoint)point {
-    id hitTestResult = self;
-
-    for (PSMTabBarCell *cell in _cells) {
-        if ([cell isHighlighted]) {
-            hitTestResult = [cell accessibilityHitTest:point];
-            break;
+    for (id child in self.accessibilityChildren) {
+        if (NSPointInRect(point, [child accessibilityFrame])) {
+            return [child accessibilityHitTest:point];
         }
     }
-
-    return hitTestResult;
+    return self;
 }
 
 #pragma mark - iTerm Add On
