@@ -4,7 +4,6 @@ set -euo pipefail
 
 xcrun xcodebuild build test \
   NSUnbufferedIO=YES \
-  -workspace iTerm2.xcworkspace \
   -scheme iTerm2 \
   -sdk macosx \
-    | xcpretty -c -f `xcpretty-travis-formatter`
+  CODE_SIGNING_REQUIRED=NO CODE_SIGN_IDENTITY=  | sed -e 's,usr/bin/clang .* -c,usr/bin/clang [redacted] -c,'
