@@ -1,5 +1,6 @@
 #include <wctype.h>
 #import "Autocomplete.h"
+#import "FindViewController.h"
 #import "iTermAdvancedSettingsModel.h"
 #import "iTermApplicationDelegate.h"
 #import "iTermCommandHistoryEntryMO+Additions.h"
@@ -128,6 +129,7 @@ const int kMaxResultContextWords = 4;
                                  includeLastNewline:NO
                              trimTrailingWhitespace:NO
                                        cappedAtSize:-1
+                                       truncateTail:YES
                                   continuationChars:nil
                                              coords:nil];
         if ([s rangeOfCharacterFromSet:nonWhitespace].location != NSNotFound) {
@@ -168,6 +170,7 @@ const int kMaxResultContextWords = 4;
                              includeLastNewline:NO
                          trimTrailingWhitespace:NO
                                    cappedAtSize:-1
+                                   truncateTail:YES
                               continuationChars:nil
                                          coords:nil];
         int maxWords = kMaxQueryContextWords;
@@ -340,8 +343,7 @@ const int kMaxResultContextWords = 4;
     more_ = YES;
     [screen setFindString:prefix_
          forwardDirection:NO
-             ignoringCase:YES
-                    regex:NO
+                     mode:iTermFindModeCaseInsensitiveSubstring
               startingAtX:x_
               startingAtY:y_
                withOffset:1
@@ -513,6 +515,7 @@ const int kMaxResultContextWords = 4;
                                              includeLastNewline:NO
                                          trimTrailingWhitespace:NO
                                                    cappedAtSize:-1
+                                                   truncateTail:YES
                                               continuationChars:nil
                                                          coords:nil];
             NSMutableString* firstWord = [NSMutableString stringWithString:immutableWord];
@@ -526,6 +529,7 @@ const int kMaxResultContextWords = 4;
                                         includeLastNewline:NO
                                     trimTrailingWhitespace:NO
                                               cappedAtSize:-1
+                                              truncateTail:YES
                                          continuationChars:nil
                                                     coords:nil];
                 if ([part length] == 0) {
@@ -563,6 +567,7 @@ const int kMaxResultContextWords = 4;
                                   includeLastNewline:NO
                               trimTrailingWhitespace:NO
                                         cappedAtSize:-1
+                                        truncateTail:YES
                                    continuationChars:nil
                                               coords:nil];
                     AcLog(@"First candidate is at %@", VT100GridWindowedRangeDescription(range));
@@ -582,6 +587,7 @@ const int kMaxResultContextWords = 4;
                                           includeLastNewline:NO
                                       trimTrailingWhitespace:NO
                                                 cappedAtSize:-1
+                                                truncateTail:YES
                                            continuationChars:nil
                                                       coords:nil];
                             if (!whitespaceBeforeCursor_) {
@@ -668,7 +674,9 @@ const int kMaxResultContextWords = 4;
     [self reloadData:YES];
     if (!populateTimer_) {
         if (self.unfilteredModel.count == 0) {
-            [self closePopupWindow];
+            dispatch_async(dispatch_get_main_queue(), ^{
+                [self closePopupWindow];
+            });
         } else {
             [self.delegate popupIsSearching:NO];
         }
