@@ -37,8 +37,7 @@
 // This used to be absurdly fast (.075) for reasons neither I nor revision
 // history can recall. This looks nicer to my eyes.
 static const float kAnimationDuration = 0.2;
-static BOOL gDefaultIgnoresCase;
-static BOOL gDefaultRegex;
+static iTermFindMode gFindMode;
 static NSString *gSearchString;
 static NSSize kFocusRingInset = { 2, 3 };
 
@@ -100,171 +99,74 @@ const CGFloat kEdgeWidth = 3;
 
 - (void)drawFocusRingMaskWithFrame:(NSRect)cellFrame inView:(NSView *)controlView {
     if (controlView.frame.origin.y >= 0) {
-        if (IsYosemiteOrLater()) {
-            [super drawFocusRingMaskWithFrame:NSInsetRect(cellFrame, kFocusRingInset.width, kFocusRingInset.height)
-                                       inView:controlView];
-        } else {
-            [super drawFocusRingMaskWithFrame:cellFrame inView:controlView];
-        }
-    }    
+        [super drawFocusRingMaskWithFrame:NSInsetRect(cellFrame, kFocusRingInset.width, kFocusRingInset.height)
+                                   inView:controlView];
+    }
 }
 
 - (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
 {
-    if (IsYosemiteOrLater()) {
-        NSRect originalFrame = cellFrame;
-        [[NSColor whiteColor] set];
-
-        BOOL focused = ([controlView respondsToSelector:@selector(currentEditor)] &&
-                        [(NSControl *)controlView currentEditor]);
-
-        CGFloat xInset, yInset;
-        if (focused) {
-            xInset = 2.5;
-            yInset = 1.5;
-        } else {
-            xInset = 0.5;
-            yInset = 0.5;
-        }
-        cellFrame = NSInsetRect(cellFrame, xInset, yInset);
-        NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:cellFrame
-                                                             xRadius:4
-                                                             yRadius:4];
-        [path fill];
-
-        if (!focused) {
-            [[NSColor colorWithCalibratedWhite:0.5 alpha:1] set];
-            [path setLineWidth:0.25];
-            [path stroke];
-
-            cellFrame = NSInsetRect(cellFrame, 0.25, 0.25);
-            path = [NSBezierPath bezierPathWithRoundedRect:cellFrame
-                                                   xRadius:4
-                                                   yRadius:4];
-            [path setLineWidth:0.25];
-            [[NSColor colorWithCalibratedWhite:0.7 alpha:1] set];
-            [path stroke];
-        }
-        [self drawInteriorWithFrame:originalFrame inView:controlView];
-        return;
-    }
-        NSColor *insetTopColor = [NSColor colorWithCalibratedWhite:1.0 alpha:0.0];
-        NSColor *insetBottomColor = [NSColor colorWithCalibratedWhite:1.0 alpha:0.35];
-        NSColor *strokeTopColor = [NSColor colorWithCalibratedWhite:0.240 alpha:1.0];
-        NSColor *strokeBottomColor = [NSColor colorWithCalibratedWhite:0.380 alpha:1.0];
-
-        if (![[controlView window] isKeyWindow]) {
-                strokeTopColor = [NSColor colorWithCalibratedWhite:0.550 alpha:1.0];
-                strokeBottomColor = [NSColor colorWithCalibratedWhite:0.557 alpha:1.0];
-        }
-
-        NSRect strokeRect = cellFrame;
-        strokeRect.size.height -= 1.0;
-        NSBezierPath *strokePath = [NSBezierPath bezierPathWithRoundedRect:strokeRect xRadius:strokeRect.size.height/2.0 yRadius:strokeRect.size.height/2.0];
-
-        NSBezierPath *insetPath = [NSBezierPath bezierPath];
-        [insetPath appendBezierPath:strokePath];
-        NSAffineTransform *transform = [NSAffineTransform transform];
-        [transform translateXBy:0 yBy:1.0];
-        [insetPath transformUsingAffineTransform:transform];
-        NSGradient *insetGradient = [[NSGradient alloc] initWithStartingColor:insetTopColor endingColor:insetBottomColor];
-        [insetGradient drawInBezierPath:insetPath angle:90.0];
-        [insetGradient release];
-
-        NSGradient *strokeGradient = [[NSGradient alloc] initWithStartingColor:strokeTopColor endingColor:strokeBottomColor];
-        [strokeGradient drawInBezierPath:strokePath angle:90.0];
-        [strokeGradient release];
-
-        NSRect fieldRect = NSInsetRect(cellFrame, 1.0, 1.0);
-        fieldRect.size.height -= 1.0;
-        NSBezierPath *fieldPath = [NSBezierPath bezierPathWithRoundedRect:fieldRect xRadius:fieldRect.size.height/2.0 yRadius:fieldRect.size.height/2.0];
-
+    NSRect originalFrame = cellFrame;
     [[NSColor whiteColor] set];
-    [fieldPath fill];
 
-    CGFloat w = fieldRect.size.width;
+    BOOL focused = ([controlView respondsToSelector:@selector(currentEditor)] &&
+                    [(NSControl *)controlView currentEditor]);
+
+    CGFloat xInset, yInset;
+    if (focused) {
+        xInset = 2.5;
+        yInset = 1.5;
+    } else {
+        xInset = 0.5;
+        yInset = 0.5;
+    }
+    cellFrame = NSInsetRect(cellFrame, xInset, yInset);
+    NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:cellFrame
+                                                         xRadius:4
+                                                         yRadius:4];
+    [path fill];
+
+    [self drawProgressBarInFrame:originalFrame path:path];
+
+    if (!focused) {
+        [[NSColor colorWithCalibratedWhite:0.5 alpha:1] set];
+        [path setLineWidth:0.25];
+        [path stroke];
+
+        cellFrame = NSInsetRect(cellFrame, 0.25, 0.25);
+        path = [NSBezierPath bezierPathWithRoundedRect:cellFrame
+                                               xRadius:4
+                                               yRadius:4];
+        [path setLineWidth:0.25];
+        [[NSColor colorWithCalibratedWhite:0.7 alpha:1] set];
+        [path stroke];
+    }
+
+    [self drawInteriorWithFrame:originalFrame inView:controlView];
+}
+
+- (void)drawProgressBarInFrame:(NSRect)cellFrame path:(NSBezierPath *)fieldPath {
     [[NSGraphicsContext currentContext] saveGraphicsState];
     [fieldPath addClip];
 
-    NSRect blueRect = NSMakeRect(0, 0, w * [self fraction] + kEdgeWidth, cellFrame.size.height);
+    const CGFloat maximumWidth = cellFrame.size.width - 1.0;
+    NSRect blueRect = NSMakeRect(0, 0, maximumWidth * [self fraction] + kEdgeWidth, cellFrame.size.height);
+    
     const CGFloat alpha = 0.3 * _alphaMultiplier;
-    NSGradient *horizontalGradient =
-        [[[NSGradient alloc] initWithStartingColor:[NSColor colorWithCalibratedRed:204.0/255.0
-                                                                             green:219.0/255.0
-                                                                              blue:233.0/255.0
-                                                                             alpha:alpha]
-                                       endingColor:[NSColor colorWithCalibratedRed:131.0/255.0
-                                                                             green:187.0/255.0
-                                                                              blue:239.0/255.0
-                                                                             alpha:alpha]] autorelease];
-    [horizontalGradient drawInRect:blueRect angle:0];
-
-    NSGradient *verticalGradient =
-        [[[NSGradient alloc] initWithStartingColor:[NSColor colorWithCalibratedRed:0/255.0
-                                                                             green:0/255.0
-                                                                              blue:0/255.0
-                                                                             alpha:alpha]
-                                       endingColor:[NSColor colorWithCalibratedRed:10.0/255.0
-                                                                             green:13.0/255.0
-                                                                              blue:0/255.0
-                                                                             alpha:alpha]] autorelease];
-    [[NSGraphicsContext currentContext] setCompositingOperation:NSCompositePlusLighter];
-    [verticalGradient drawInRect:blueRect angle:90];
-
-    NSGradient *edgeGradient =
-        [[[NSGradient alloc] initWithStartingColor:[NSColor colorWithCalibratedRed:255/255.0
-                                                                             green:255/255.0
-                                                                              blue:255/255.0
-                                                                             alpha:0.0]
-                                       endingColor:[NSColor colorWithCalibratedRed:255.0/255.0
-                                                                             green:255.0/255.0
-                                                                              blue:255.0/255.0
-                                                                             alpha:1.0]] autorelease];
-    [[NSGraphicsContext currentContext] setCompositingOperation:NSCompositeSourceOver];
-    NSRect edgeRect = NSMakeRect(blueRect.size.width - kEdgeWidth, 0, kEdgeWidth, blueRect.size.height);
-    [edgeGradient drawInRect:edgeRect angle:0];
+    [[NSColor colorWithCalibratedRed:0.6
+                               green:0.6
+                               blue:1.0
+                               alpha:alpha] set];
+    NSRectFillUsingOperation(blueRect, NSCompositingOperationSourceOver);
 
     [[NSGraphicsContext currentContext] restoreGraphicsState];
-
-        // Draw the inner shadow
-        [[NSGraphicsContext currentContext] saveGraphicsState];
-        NSShadow *innerShadow = [[NSShadow alloc] init];
-        float innerShadowAlpha = 0.4;
-        if (![[controlView window] isKeyWindow])
-                innerShadowAlpha = 0.2;
-        [innerShadow setShadowColor:[NSColor colorWithCalibratedWhite:0.0 alpha:innerShadowAlpha]];
-        [innerShadow setShadowOffset:NSMakeSize(0, -1.0)];
-        [innerShadow setShadowBlurRadius:1.0];
-        [innerShadow set];
-
-        [fieldPath addClip];
-
-        NSBezierPath *outlinePath = [NSBezierPath bezierPath];
-        [outlinePath appendBezierPath:strokePath];
-        [outlinePath appendBezierPath:fieldPath];
-        [outlinePath setWindingRule:NSEvenOddWindingRule];
-        [strokeTopColor set];
-        [outlinePath fill];
-
-        [[NSGraphicsContext currentContext] restoreGraphicsState];
-        [innerShadow release];
-
-        [self drawInteriorWithFrame:cellFrame inView:controlView];
-        if ([controlView respondsToSelector:@selector(currentEditor)] && [(NSControl *)controlView currentEditor]) {
-                [[NSGraphicsContext currentContext] saveGraphicsState];
-                NSSetFocusRingStyle(NSFocusRingOnly);
-                [strokePath fill];
-                [[NSGraphicsContext currentContext] restoreGraphicsState];
-        }
 }
 
-
 @end
 
 @interface FindState : NSObject
 
-@property(nonatomic, assign) BOOL ignoreCase;
-@property(nonatomic, assign) BOOL regex;
+@property(nonatomic, assign) iTermFindMode mode;
 @property(nonatomic, copy) NSString *string;
 
 @end
@@ -291,10 +193,6 @@ const CGFloat kEdgeWidth = 3;
 
 @implementation FindViewController {
     IBOutlet NSSearchField* findBarTextField_;
-    // These pointers are just "prototypes" and do not refer to any actual menu
-    // items.
-    IBOutlet NSMenuItem* ignoreCaseMenuItem_;
-    IBOutlet NSMenuItem* regexMenuItem_;
 
     FindState *savedState_;
     FindState *state_;
@@ -323,27 +221,43 @@ const CGFloat kEdgeWidth = 3;
 }
 
 
-+ (void)initialize
-{
-    gDefaultIgnoresCase =
-        [[NSUserDefaults standardUserDefaults] objectForKey:@"findIgnoreCase_iTerm"] ?
-            [[NSUserDefaults standardUserDefaults] boolForKey:@"findIgnoreCase_iTerm"] :
-            YES;
-    gDefaultRegex = [[NSUserDefaults standardUserDefaults] boolForKey:@"findRegex_iTerm"];
++ (void)initialize {
+    NSNumber *mode = [[NSUserDefaults standardUserDefaults] objectForKey:@"findMode_iTerm"];
+    if (!mode) {
+        // Migrate legacy value.
+        NSNumber *ignoreCase = [[NSUserDefaults standardUserDefaults] objectForKey:@"findIgnoreCase_iTerm"];
+        BOOL caseSensitive = ignoreCase ? ![ignoreCase boolValue] : NO;
+        BOOL isRegex = [[NSUserDefaults standardUserDefaults] boolForKey:@"findRegex_iTerm"];
+
+        if (caseSensitive && isRegex) {
+            gFindMode = iTermFindModeCaseSensitiveRegex;
+        } else if (!caseSensitive && isRegex) {
+            gFindMode = iTermFindModeCaseInsensitiveRegex;
+        } else if (caseSensitive && !isRegex) {
+            gFindMode = iTermFindModeCaseSensitiveSubstring;
+        } else if (!caseSensitive && !isRegex) {
+            gFindMode = iTermFindModeSmartCaseSensitivity;  // Upgrade case-insensitive substring to smart case sensitivity.
+        }
+    } else {
+        // Modern value
+        gFindMode = [mode unsignedIntegerValue];
+    }
 }
 
 - (instancetype)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
     self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
     if (self) {
+        ITERM_IGNORE_PARTIAL_BEGIN
         [findBarTextField_ setDelegate:self];
+        ITERM_IGNORE_PARTIAL_END
         state_ = [[FindState alloc] init];
-        state_.ignoreCase = gDefaultIgnoresCase;
-        state_.regex = gDefaultRegex;
+        state_.mode = gFindMode;
         [[NSNotificationCenter defaultCenter] addObserver:self
                                                  selector:@selector(_loadFindStringFromSharedPasteboard)
                                                      name:@"iTermLoadFindStringFromSharedPasteboard"
                                                    object:nil];
         [self loadView];
+        self.view.wantsLayer = [iTermAdvancedSettingsModel useMetal];
         [[self view] setHidden:YES];
     }
     return self;
@@ -398,16 +312,14 @@ const CGFloat kEdgeWidth = 3;
     [self close];
 }
 
-- (NSRect)collapsedFrame
-{
+- (NSRect)collapsedFrame {
     return NSMakeRect([[self view] frame].origin.x,
                       fullFrame_.origin.y + [self superframe].size.height + fullFrame_.size.height,
                       [[self view] frame].size.width,
                       0);
 }
 
-- (NSRect)fullSizeFrame
-{
+- (NSRect)fullSizeFrame {
     return NSMakeRect([[self view] frame].origin.x,
                       fullFrame_.origin.y + [self superframe].size.height,
                       [[self view] frame].size.width,
@@ -427,12 +339,14 @@ const CGFloat kEdgeWidth = 3;
     [[NSAnimationContext currentContext] setCompletionHandler:^{
         [[self view] setHidden:YES];
         [[[[self view] window] contentView] setNeedsDisplay:YES];
+        _isVisible = NO;
+        [self.delegate findViewControllerVisibilityDidChange:self];
     }];
     [[NSAnimationContext currentContext] setDuration:kAnimationDuration];
     [[[self view] animator] setFrame:[self collapsedFrame]];
     [NSAnimationContext endGrouping];
 
-    [delegate_ clearHighlights];
+    [delegate_ findViewControllerClearSearch];
     [delegate_ findViewControllerMakeDocumentFirstResponder];
 }
 
@@ -446,18 +360,25 @@ const CGFloat kEdgeWidth = 3;
     [savedState_ release];
     savedState_ = state_;
     state_ = [[FindState alloc] init];
-    state_.ignoreCase = savedState_.ignoreCase;
-    state_.regex = savedState_.regex;
+    state_.mode = savedState_.mode;
     state_.string = savedState_.string;
 }
 
 - (void)open {
+    if ([findBarTextField_.window.appearance.name isEqual:NSAppearanceNameVibrantDark]) {
+        findBarTextField_.appearance = [NSAppearance appearanceNamed:NSAppearanceNameVibrantLight];
+    } else {
+        findBarTextField_.appearance = nil;
+    }
+
     if (savedState_) {
         [self restoreState];
-        ignoreCaseMenuItem_.state = state_.ignoreCase ? NSOnState : NSOffState;
-        regexMenuItem_.state = state_.regex ? NSOnState : NSOffState;
         findBarTextField_.stringValue = state_.string;
     }
+
+    _isVisible = YES;
+    [self.delegate findViewControllerVisibilityDidChange:self];
+
     [[self view] setFrame:[self collapsedFrame]];
     [[self view] setHidden:NO];
     [[NSAnimationContext currentContext] setDuration:kAnimationDuration];
@@ -493,8 +414,12 @@ const CGFloat kEdgeWidth = 3;
 }
 
 - (void)setProgress:(double)progress {
-    [findBarTextField_.cell setFraction:progress];
-    iTermSearchFieldCell *cell = findBarTextField_.cell;
+    iTermSearchFieldCell *cell = (iTermSearchFieldCell *)findBarTextField_.cell;
+    if (round(progress * 100) != round(cell.fraction * 100)) {
+        [findBarTextField_ setNeedsDisplay:YES];
+    }
+
+    [cell setFraction:progress];
     if (cell.needsAnimation && !_animationTimer) {
         _animationTimer = [NSTimer scheduledTimerWithTimeInterval:1/60.0
                                                   target:self
@@ -514,8 +439,7 @@ const CGFloat kEdgeWidth = 3;
     [findBarTextField_ setNeedsDisplay:YES];
 }
 
-- (void)_continueSearch
-{
+- (BOOL)_continueSearch {
     BOOL more = NO;
     if ([delegate_ findInProgress]) {
         double progress;
@@ -527,10 +451,10 @@ const CGFloat kEdgeWidth = 3;
         timer_ = nil;
         [self setProgress:1];
     }
+    return more;
 }
 
-- (void)_setSearchString:(NSString *)s
-{
+- (void)_setSearchString:(NSString *)s {
     if (!savedState_) {
         [gSearchString autorelease];
         gSearchString = [s retain];
@@ -538,58 +462,47 @@ const CGFloat kEdgeWidth = 3;
     }
 }
 
-- (void)_setIgnoreCase:(BOOL)set
-{
+- (void)setMode:(iTermFindMode)set {
     if (!savedState_) {
-        gDefaultIgnoresCase = set;
-        [[NSUserDefaults standardUserDefaults] setBool:set
-                                                forKey:@"findIgnoreCase_iTerm"];
+        gFindMode = set;
+        // The user defaults key got recycled to make it clear whether the legacy (number) or modern value (dict) is
+        // in use, but the key doesn't reflect its true meaning any more.
+        [[NSUserDefaults standardUserDefaults] setObject:@(set) forKey:@"findMode_iTerm"];
     }
 }
 
-- (void)_setRegex:(BOOL)set
-{
-    if (!savedState_) {
-        gDefaultRegex = set;
-        [[NSUserDefaults standardUserDefaults] setBool:set
-                                                forKey:@"findRegex_iTerm"];
-    }
-}
-
-- (void)_setSearchDefaults
-{
+- (void)_setSearchDefaults {
     [self _setSearchString:[findBarTextField_ stringValue]];
-    [self _setIgnoreCase:state_.ignoreCase];
-    [self _setRegex:state_.regex];
+    [self setMode:state_.mode];
 }
 
 - (void)findSubString:(NSString *)subString
      forwardDirection:(BOOL)direction
-         ignoringCase:(BOOL)ignoringCase
-                regex:(BOOL)regex
-           withOffset:(int)offset
-{
+                 mode:(iTermFindMode)mode
+           withOffset:(int)offset {
     BOOL ok = NO;
     if ([delegate_ canSearch]) {
         if ([subString length] <= 0) {
-            [delegate_ clearHighlights];
+            [delegate_ findViewControllerClearSearch];
         } else {
             [delegate_ findString:subString
                  forwardDirection:direction
-                     ignoringCase:ignoringCase
-                            regex:regex
+                             mode:mode
                        withOffset:offset];
             ok = YES;
         }
     }
 
     if (ok && !timer_) {
-        timer_ = [NSTimer scheduledTimerWithTimeInterval:0.01
-                                                  target:self
-                                                selector:@selector(_continueSearch)
-                                                userInfo:nil
-                                                 repeats:YES];
         [self setProgress:0];
+        if ([self _continueSearch]) {
+            timer_ = [NSTimer scheduledTimerWithTimeInterval:0.01
+                                                      target:self
+                                                    selector:@selector(_continueSearch)
+                                                    userInfo:nil
+                                                     repeats:YES];
+            [[NSRunLoop currentRunLoop] addTimer:timer_ forMode:NSRunLoopCommonModes];
+        }
     } else if (!ok && timer_) {
         [timer_ invalidate];
         timer_ = nil;
@@ -597,28 +510,23 @@ const CGFloat kEdgeWidth = 3;
     }
 }
 
-- (void)searchNext
-{
+- (void)searchNext {
     [self _setSearchDefaults];
     [self findSubString:savedState_ ? state_.string : gSearchString
        forwardDirection:YES
-           ignoringCase:state_.ignoreCase
-                  regex:state_.regex
+                   mode:state_.mode
              withOffset:1];
 }
 
-- (void)searchPrevious
-{
+- (void)searchPrevious {
     [self _setSearchDefaults];
     [self findSubString:savedState_ ? state_.string : gSearchString
        forwardDirection:NO
-           ignoringCase:state_.ignoreCase
-                  regex:state_.regex
+                   mode:state_.mode
              withOffset:1];
 }
 
-- (IBAction)searchNextPrev:(id)sender
-{
+- (IBAction)searchNextPrev:(id)sender {
     if ([sender selectedSegment] == 0) {
         [self searchPrevious];
     } else {
@@ -628,30 +536,18 @@ const CGFloat kEdgeWidth = 3;
              forSegment:[sender selectedSegment]];
 }
 
-- (BOOL)validateUserInterfaceItem:(NSMenuItem*)item
-{
-    if ([item action] == @selector(toggleIgnoreCase:)) {
-        [item setState:(state_.ignoreCase ? NSOnState : NSOffState)];
-    } else if ([item action] == @selector(toggleRegex:)) {
-        [item setState:(state_.regex ? NSOnState : NSOffState)];
-    }
+- (BOOL)validateUserInterfaceItem:(NSMenuItem *)item {
+    item.state = (item.tag == state_.mode) ? NSOnState : NSOffState;
     return YES;
 }
 
-- (IBAction)toggleIgnoreCase:(id)sender
-{
-    state_.ignoreCase = !state_.ignoreCase;
-    [self _setIgnoreCase:state_.ignoreCase];
+- (IBAction)changeMode:(id)sender {
+    state_.mode = (iTermFindMode)[sender tag];
+    [self setMode:state_.mode];
 }
 
-- (IBAction)toggleRegex:(id)sender
-{
-    state_.regex = !state_.regex;
-    [self _setRegex:state_.regex];
-}
 
-- (void)_loadFindStringIntoSharedPasteboard
-{
+- (void)_loadFindStringIntoSharedPasteboard {
     if (savedState_) {
         return;
     }
@@ -663,28 +559,24 @@ const CGFloat kEdgeWidth = 3;
     }
 }
 
-- (void)setFindString:(NSString*)string
-{
+- (void)setFindString:(NSString*)string {
     [findBarTextField_ setStringValue:string];
     [self _loadFindStringIntoSharedPasteboard];
 }
 
 - (void)closeViewAndDoTemporarySearchForString:(NSString *)string
-                                 ignoringCase:(BOOL)ignoringCase
-                                        regex:(BOOL)regex {
+                                          mode:(iTermFindMode)mode {
     [self close];
     if (!savedState_) {
         [self saveState];
     }
-    state_.ignoreCase = ignoringCase;
-    state_.regex = regex;
+    state_.mode = mode;
     state_.string = string;
     findBarTextField_.stringValue = string;
     [self doSearch];
 }
 
-- (void)controlTextDidChange:(NSNotification *)aNotification
-{
+- (void)controlTextDidChange:(NSNotification *)aNotification {
     NSTextField *field = [aNotification object];
     if (field != findBarTextField_) {
         return;
@@ -840,23 +732,20 @@ const CGFloat kEdgeWidth = 3;
     [self _setSearchDefaults];
     [self findSubString:theString
        forwardDirection:NO
-           ignoringCase:state_.ignoreCase
-                  regex:state_.regex
-             withOffset:0];
+                   mode:state_.mode
+             withOffset:-1];
 }
 
-- (void)deselectFindBarTextField
-{
+- (void)deselectFindBarTextField {
     NSText* fieldEditor = [[[self view] window] fieldEditor:YES
                                                   forObject:findBarTextField_];
     [fieldEditor setSelectedRange:NSMakeRange([[fieldEditor string] length], 0)];
     [fieldEditor setNeedsDisplay:YES];
 }
 
-- (BOOL)control:(NSControl*)control
-       textView:(NSTextView*)textView
-    doCommandBySelector:(SEL)commandSelector
-{
+- (BOOL)control:(NSControl *)control
+       textView:(NSTextView *)textView
+    doCommandBySelector:(SEL)commandSelector {
     if (control != findBarTextField_) {
         return NO;
     }
@@ -923,13 +812,11 @@ const CGFloat kEdgeWidth = 3;
     return;
 }
 
-- (void)setDelegate:(id<FindViewControllerDelegate>)delegate
-{
+- (void)setDelegate:(id<FindViewControllerDelegate>)delegate {
     delegate_ = delegate;
 }
 
-- (id<FindViewControllerDelegate>)delegate
-{
+- (id<FindViewControllerDelegate>)delegate {
     return delegate_;
 }
 
