@@ -1,18 +1,15 @@
-// Debug option
-#define PtyTaskDebugLog(fmt, ...)
-// Use this instead to debug this module:
-// #define PtyTaskDebugLog NSLog
-
 #define MAXRW 1024
 
 #import "Coprocess.h"
 #import "DebugLogging.h"
+#import "iTermGrowlDelegate.h"
 #import "NSWorkspace+iTerm.h"
 #import "PreferencePanel.h"
 #import "ProcessCache.h"
 #import "PTYTask.h"
 #import "TaskNotifier.h"
 #import "iTermAdvancedSettingsModel.h"
+#import "iTermLSOF.h"
 #import "iTermOrphanServerAdopter.h"
 #import <OpenDirectory/OpenDirectory.h>
 
@@ -35,6 +32,7 @@
 #include <util.h>
 
 #define CTRLKEY(c) ((c)-'A'+1)
+const int kNumFileDescriptorsToDup = NUM_FILE_DESCRIPTORS_TO_PASS_TO_SERVER;
 
 NSString *kCoprocessStatusChangeNotification = @"kCoprocessStatusChangeNotification";
 
@@ -111,6 +109,10 @@ setup_tty_param(struct termios* term,
     BOOL _paused;
 
     int _socketFd;  // File descriptor for unix domain socket connected to server. Only safe to close after server is dead.
+    
+    VT100GridSize _desiredSize;
+    NSTimeInterval _timeOfLastSizeChange;
+    BOOL _rateLimitedSetSizeToDesiredSizePending;
 }
 
 - (instancetype)init {
@@ -187,8 +189,7 @@ static void HandleSigChld(int n) {
     UnblockTaskNotifier();
 }
 
-- (NSString *)command
-{
+- (NSString *)originalCommand {
     return command_;
 }
 
@@ -271,42 +272,48 @@ static void HandleSigChld(int n) {
     [[TaskNotifier sharedInstance] registerTask:self];
 }
 
-// Like login_tty but makes fd 0 the master, fd 1 the slave, and fd 2 an open unix-domain socket
-// for transferring file descriptors.
-static void MyLoginTTY(int master, int slave, int serverSocketFd) {
+// Like login_tty but makes fd 0 the master, fd 1 the slave, fd 2 an open unix-domain socket
+// for transferring file descriptors, and fd 3 the write end of a pipe that closes when the server
+// dies.
+// IMPORTANT: This runs between fork and exec. Careful what you do.
+static void MyLoginTTY(int master, int slave, int serverSocketFd, int deadMansPipeWriteEnd) {
     setsid();
     ioctl(slave, TIOCSCTTY, NULL);
 
     // This array keeps track of which file descriptors are in use and should not be dup2()ed over.
-    // It has |inuseCount| valid elements.
-    int inuse[9] = { 0, 1, 2, master, slave, serverSocketFd, -1, -1, -1 };
-    int inuseCount = 6;
+    // It has |inuseCount| valid elements. inuse must have inuseCount + arraycount(orig) elements.
+    int inuse[3 * kNumFileDescriptorsToDup] = {
+       0, 1, 2, 3,  // FDs get duped to the lowest numbers so reserve them
+       master, slave, serverSocketFd, deadMansPipeWriteEnd,  // FDs to get duped, which mustn't be overwritten
+       -1, -1, -1, -1 };  // Space for temp values to ensure they don't get resused
+    int inuseCount = 2 * kNumFileDescriptorsToDup;
 
     // File descriptors get dup2()ed to temporary numbers first to avoid stepping on each other or
     // on any of the desired final values. Their temporary values go in here. The first is always
     // master, then slave, then server socket.
-    int temp[3];
+    int temp[kNumFileDescriptorsToDup];
 
     // The original file descriptors to renumber.
-    int orig[3] = { master, slave, serverSocketFd };
+    int orig[kNumFileDescriptorsToDup] = { master, slave, serverSocketFd, deadMansPipeWriteEnd };
 
     for (int o = 0; o < sizeof(orig) / sizeof(*orig); o++) {  // iterate over orig
         int original = orig[o];
 
-        // Try to find a temp value that doesn't belong to inuse
-        for (int t = 0; t < sizeof(inuse) / sizeof(*inuse); t++) {
+        // Try to find a candidate file descriptor that is not important to us (i.e., does not belong
+        // to the inuse array).
+        for (int candidate = 0; candidate < sizeof(inuse) / sizeof(*inuse); candidate++) {
             BOOL isInUse = NO;
             for (int i = 0; i < sizeof(inuse) / sizeof(*inuse); i++) {
-                if (inuse[i] == t) {
+                if (inuse[i] == candidate) {
                     isInUse = YES;
                     break;
                 }
             }
             if (!isInUse) {
                 // t is good. dup orig[o] to t and close orig[o]. Save t in temp[o].
-                inuse[inuseCount++] = t;
-                temp[o] = t;
-                dup2(original, t);
+                inuse[inuseCount++] = candidate;
+                temp[o] = candidate;
+                dup2(original, candidate);
                 close(original);
                 break;
             }
@@ -326,16 +333,19 @@ static int MyForkPty(int *amaster,
                      char *name,
                      struct termios *termp,
                      struct winsize *winp,
-                     int serverSocketFd) {
+                     int serverSocketFd,
+                     int deadMansPipeWriteEnd) {
     assert([iTermAdvancedSettingsModel runJobsInServers]);
     int master;
     int slave;
 
+    iTermFileDescriptorServerLog("Calling openpty");
     if (openpty(&master, &slave, name, termp, winp) == -1) {
         NSLog(@"openpty failed: %s", strerror(errno));
         return -1;
     }
 
+    iTermFileDescriptorServerLog("Calling fork");
     pid_t pid = fork();
     switch (pid) {
         case -1:
@@ -345,13 +355,15 @@ static int MyForkPty(int *amaster,
 
         case 0:
             // child
-            MyLoginTTY(master, slave, serverSocketFd);
+            MyLoginTTY(master, slave, serverSocketFd, deadMansPipeWriteEnd);
             return 0;
 
         default:
             // parent
             *amaster = master;
             close(slave);
+            close(serverSocketFd);
+            close(deadMansPipeWriteEnd);
             return pid;
     }
 }
@@ -410,6 +422,18 @@ static int MyForkPty(int *amaster,
     return newEnvironment;
 }
 
+- (BOOL)passwordInput {
+    struct termios termAttributes;
+    if ([iTermAdvancedSettingsModel detectPasswordInput] &&
+        fd > 0 &&
+        isatty(fd) &&
+        tcgetattr(fd, &termAttributes) == 0) {
+        return !(termAttributes.c_lflag & ECHO) && (termAttributes.c_lflag & ICANON);
+    } else {
+        return NO;
+    }
+}
+
 - (void)launchWithPath:(NSString *)progpath
              arguments:(NSArray *)args
            environment:(NSDictionary *)env
@@ -420,6 +444,9 @@ static int MyForkPty(int *amaster,
     struct winsize win;
     char theTtyname[PATH_MAX];
 
+    [command_ autorelease];
+    command_ = [progpath copy];
+
     env = [self environmentBySettingShell:env];
     if ([iTermAdvancedSettingsModel runJobsInServers]) {
         // We want to run
@@ -436,8 +463,6 @@ static int MyForkPty(int *amaster,
         progpath = iterm2Binary;
     }
 
-    [command_ autorelease];
-    command_ = [progpath copy];
     path = [progpath copy];
 
     setup_tty_param(&term, &win, width, height, isUTF8);
@@ -471,15 +496,27 @@ static int MyForkPty(int *amaster,
     DLog(@"Preparing to launch a job. Command is %@ and args are %@", commandToExec, args);
     DLog(@"Environment is\n%@", env);
     char **newEnviron = [self environWithOverrides:env];
-
+    int deadMansPipe[2] = { 0, 0 };
+    
     // Note: stringByStandardizingPath will automatically call stringByExpandingTildeInPath.
     const char *initialPwd = [[[env objectForKey:@"PWD"] stringByStandardizingPath] UTF8String];
     pid_t pid;
     int connectionFd = -1;
+    int numFileDescriptorsToPreserve;
+    BOOL closeFileDescriptors = YES;
     if ([iTermAdvancedSettingsModel runJobsInServers]) {
+        closeFileDescriptors = NO;
         // Create a temporary filename for the unix domain socket. It'll only exist for a moment.
         NSString *tempPath = [[NSWorkspace sharedWorkspace] temporaryFileNameWithPrefix:@"iTerm2-temp-socket."
                                                                                  suffix:@""];
+        if (tempPath == nil) {
+            NSAlert *alert = [[[NSAlert alloc] init] autorelease];
+            alert.messageText = @"Error";
+            alert.informativeText = [NSString stringWithFormat:@"An error was encountered while creating a temporary file with mkstemps. Verify that %@ exists and is writable.", NSTemporaryDirectory()];
+            [alert addButtonWithTitle:@"OK"];
+            [alert runModal];
+            return;
+        }
 
         // Begin listening on that path as a unix domain socket.
         int serverSocketFd = iTermFileDescriptorServerSocketBindListen(tempPath.UTF8String);
@@ -491,20 +528,26 @@ static int MyForkPty(int *amaster,
         // In another thread, accept on the unix domain socket. Since it's
         // already listening, there's no race here. connect will block until
         // accept is called if the main thread wins the race. accept will block
-        // til connect is called if the background thread wins the race. 
+        // til connect is called if the background thread wins the race.
+        iTermFileDescriptorServerLog("Kicking off a background job to accept() in the server");
         dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
+            iTermFileDescriptorServerLog("Now running the accept queue block");
             serverConnectionFd = iTermFileDescriptorServerAccept(serverSocketFd);
 
             // Let the main thread go. This is necessary to ensure that
             // serverConnectionFd is written to before the main thread uses it.
+            iTermFileDescriptorServerLog("Signal the semaphore");
             dispatch_semaphore_signal(semaphore);
         });
 
         // Connect to the server running in a thread.
         connectionFd = iTermFileDescriptorClientConnect(tempPath.UTF8String);
+        assert(connectionFd != -1);  // If this happens the block dispatched above never returns. Ran out of FDs, presumably.
 
         // Wait for serverConnectionFd to be written to.
+        iTermFileDescriptorServerLog("Waiting for the semaphore");
         dispatch_semaphore_wait(semaphore, DISPATCH_TIME_FOREVER);
+        iTermFileDescriptorServerLog("The semaphore was signaled");
 
         dispatch_release(semaphore);
 
@@ -516,11 +559,18 @@ static int MyForkPty(int *amaster,
 
         // Now fork. This variant of forkpty passes through the master, slave,
         // and serverConnectionFd to the child job.
-        pid = _serverPid = MyForkPty(&fd, theTtyname, &term, &win, serverConnectionFd);
+        pipe(deadMansPipe);
+        
+        // This closes serverConnectionFd and deadMansPipe[1] in the parent process but not the child.
+        iTermFileDescriptorServerLog("Calling MyForkPty");
+        pid = _serverPid = MyForkPty(&fd, theTtyname, &term, &win, serverConnectionFd, deadMansPipe[1]);
+        numFileDescriptorsToPreserve = kNumFileDescriptorsToDup;
     } else {
         pid = _childPid = forkpty(&fd, theTtyname, &term, &win);
+        numFileDescriptorsToPreserve = 3;
     }
     if (pid == (pid_t)0) {
+        // Child
         // Do not start the new process with a signal handler.
         signal(SIGCHLD, SIG_DFL);
         signal(SIGPIPE, SIG_DFL);
@@ -531,8 +581,12 @@ static int MyForkPty(int *amaster,
 
         // Apple opens files without the close-on-exec flag (e.g., Extras2.rsrc).
         // See issue 2662.
-        for (int j = 3; j < getdtablesize(); j++) {
-            close(j);
+        if (closeFileDescriptors) {
+            // If running jobs in servers close file descriptors after exec when it's safe to
+            // enumerate files in /dev/fd. This is the potentially very slow path (issue 5391).
+            for (int j = numFileDescriptorsToPreserve; j < getdtablesize(); j++) {
+                close(j);
+            }
         }
 
         chdir(initialPwd);
@@ -550,12 +604,10 @@ static int MyForkPty(int *amaster,
         sleep(1);
         _exit(-1);
     } else if (pid < (pid_t)0) {
-        PtyTaskDebugLog(@"%@ %s", progpath, strerror(errno));
-        NSRunCriticalAlertPanel(@"Unable to Fork!",
-                                @"iTerm cannot launch the program for this session.",
-                                @"OK",
-                                nil,
-                                nil);
+        // Error
+        DLog(@"Unable to fork %@: %s", progpath, strerror(errno));
+        [[iTermGrowlDelegate sharedInstance] growlNotify:@"Unable to fork!" withDescription:@"You may have too many processes already running."];
+        
         for (int j = 0; newEnviron[j]; j++) {
             free(newEnviron[j]);
         }
@@ -576,7 +628,8 @@ static int MyForkPty(int *amaster,
         // really need the rest of the stuff in serverConnection since we already know
         // it, but that's ok.
         iTermFileDescriptorServerConnection serverConnection =
-            iTermFileDescriptorClientRead(connectionFd);
+            iTermFileDescriptorClientRead(connectionFd, deadMansPipe[0]);
+        close(deadMansPipe[0]);
         if (serverConnection.ok) {
             // We intentionally leave connectionFd open. If iTerm2 stops unexpectedly then its closure
             // lets the server know it should call accept(). We now have two copies of the master PTY
@@ -597,7 +650,7 @@ static int MyForkPty(int *amaster,
         } else {
             close(fd);
             NSLog(@"Server died immediately!");
-            [_delegate brokenPipe];
+            [_delegate taskDiedImmediately];
         }
     } else {
         // Jobs are direct children of iTerm2
@@ -775,8 +828,11 @@ static int MyForkPty(int *amaster,
     [self.delegate threadedTaskBrokenPipe];
 }
 
-- (void)sendSignal:(int)signo {
-    if (_serverChildPid != -1) {
+- (void)sendSignal:(int)signo toServer:(BOOL)toServer {
+    if (toServer && _serverPid != -1) {
+        DLog(@"Sending signal to server %@", @(_serverPid));
+        kill(_serverPid, signo);
+    } else if (_serverChildPid != -1) {
         kill(_serverChildPid, signo);
      } else if (_childPid >= 0) {
          kill(_childPid, signo);
@@ -805,16 +861,45 @@ static int MyForkPty(int *amaster,
 }
 
 - (void)setSize:(VT100GridSize)size {
-    PtyTaskDebugLog(@"Set terminal size to %@", VT100GridSizeDescription(size));
-    struct winsize winsize;
+    DLog(@"Set terminal size to %@", VT100GridSizeDescription(size));
     if (self.fd == -1) {
         return;
     }
 
+    _desiredSize = size;
+    [self rateLimitedSetSizeToDesiredSize];
+}
+
+- (void)rateLimitedSetSizeToDesiredSize {
+    if (_rateLimitedSetSizeToDesiredSizePending) {
+        return;
+    }
+
+    static const NSTimeInterval kDelayBetweenSizeChanges = 0.2;
+    if ([NSDate timeIntervalSinceReferenceDate] - _timeOfLastSizeChange < kDelayBetweenSizeChanges) {
+        // Avoid problems with signal coalescing of SIGWINCH preventing redraw for the second size
+        // change. For example, issue 5096 and 4494.
+        _rateLimitedSetSizeToDesiredSizePending = YES;
+        DLog(@" ** Rate limiting **");
+        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(kDelayBetweenSizeChanges * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
+            _rateLimitedSetSizeToDesiredSizePending = NO;
+            [self setTerminalSizeToDesiredSize];
+        });
+    } else {
+        [self setTerminalSizeToDesiredSize];
+    }
+}
+
+- (void)setTerminalSizeToDesiredSize {
+    DLog(@"Set size of %@ to %@", _delegate, VT100GridSizeDescription(_desiredSize));
+    _timeOfLastSizeChange = [NSDate timeIntervalSinceReferenceDate];
+    
+    struct winsize winsize;
     ioctl(fd, TIOCGWINSZ, &winsize);
-    if (winsize.ws_col != size.width || winsize.ws_row != size.height) {
-        winsize.ws_col = size.width;
-        winsize.ws_row = size.height;
+    if (winsize.ws_col != _desiredSize.width || winsize.ws_row != _desiredSize.height) {
+        DLog(@"Actually setting the size");
+        winsize.ws_col = _desiredSize.width;
+        winsize.ws_row = _desiredSize.height;
         ioctl(fd, TIOCSWINSZ, &winsize);
     }
 }
@@ -849,7 +934,7 @@ static int MyForkPty(int *amaster,
 - (void)stop {
     self.paused = NO;
     [self stopLogging];
-    [self sendSignal:SIGHUP];
+    [self sendSignal:SIGHUP toServer:NO];
     [self killServerIfRunning];
 
     if (fd >= 0) {
@@ -975,11 +1060,11 @@ static int MyForkPty(int *amaster,
     pid_t oldestPid = -1;
     for (int i = 0; i < numPids; ++i) {
         struct proc_taskallinfo taskAllInfo;
-        int rc = proc_pidinfo(pids[i],
-                              PROC_PIDTASKALLINFO,
-                              0,
-                              &taskAllInfo,
-                              sizeof(taskAllInfo));
+        int rc = iTermProcPidInfoWrapper(pids[i],
+                                         PROC_PIDTASKALLINFO,
+                                         0,
+                                         &taskAllInfo,
+                                         sizeof(taskAllInfo));
         if (rc <= 0) {
             continue;
         }
@@ -1008,6 +1093,10 @@ static int MyForkPty(int *amaster,
 }
 
 - (NSString*)getWorkingDirectory {
+    if (self.pid == -1) {
+        DLog(@"Want to use the kernel to get the working directory but pid = -1");
+        return nil;
+    }
     DLog(@"Using OS magic to get the working directory");
     struct proc_vnodepathinfo vpi;
     int ret;
@@ -1015,13 +1104,13 @@ static int MyForkPty(int *amaster,
     // This only works if the child process is owned by our uid
     // Notably it seems to work (at least on 10.10) even if the process ID is
     // not owned by us.
-    ret = proc_pidinfo(self.pid, PROC_PIDVNODEPATHINFO, 0, &vpi, sizeof(vpi));
+    ret = iTermProcPidInfoWrapper(self.pid, PROC_PIDVNODEPATHINFO, 0, &vpi, sizeof(vpi));
     if (ret <= 0) {
         // The child was probably owned by root (which is expected if it's
         // a login shell. Use the cwd of its oldest child instead.
         pid_t childPid = [self getFirstChildOfPid:self.pid];
         if (childPid > 0) {
-            ret = proc_pidinfo(childPid, PROC_PIDVNODEPATHINFO, 0, &vpi, sizeof(vpi));
+            ret = iTermProcPidInfoWrapper(childPid, PROC_PIDVNODEPATHINFO, 0, &vpi, sizeof(vpi));
         }
     }
     if (ret <= 0) {
