@@ -1,5 +1,5 @@
 #import <Cocoa/Cocoa.h>
-#import "CharacterRun.h"
+#import "FindViewController.h"
 #import "ITAddressBookMgr.h"
 #import "iTerm.h"
 #import "iTermColorMap.h"
@@ -51,7 +51,16 @@ typedef NS_ENUM(NSInteger, PTYTextViewSelectionEndpoint) {
 
 typedef NS_ENUM(NSInteger, PTYTextViewSelectionExtensionDirection) {
     kPTYTextViewSelectionExtensionDirectionLeft,
-    kPTYTextViewSelectionExtensionDirectionRight
+    kPTYTextViewSelectionExtensionDirectionRight,
+
+    // These ignore the unit and are simple movements.
+    kPTYTextViewSelectionExtensionDirectionUp,
+    kPTYTextViewSelectionExtensionDirectionDown,
+    kPTYTextViewSelectionExtensionDirectionStartOfLine,
+    kPTYTextViewSelectionExtensionDirectionEndOfLine,
+    kPTYTextViewSelectionExtensionDirectionTop,
+    kPTYTextViewSelectionExtensionDirectionBottom,
+    kPTYTextViewSelectionExtensionDirectionStartOfIndentation,
 };
 
 typedef NS_ENUM(NSInteger, PTYTextViewSelectionExtensionUnit) {
@@ -64,6 +73,7 @@ typedef NS_ENUM(NSInteger, PTYTextViewSelectionExtensionUnit) {
 @protocol PTYTextViewDelegate <NSObject>
 
 - (BOOL)xtermMouseReporting;
+- (BOOL)xtermMouseReportingAllowMouseWheel;
 - (BOOL)isPasting;
 - (void)queueKeyDown:(NSEvent *)event;
 - (void)keyDown:(NSEvent *)event;
@@ -125,6 +135,7 @@ typedef NS_ENUM(NSInteger, PTYTextViewSelectionExtensionUnit) {
 - (BOOL)textViewHasCoprocess;
 - (void)textViewPostTabContentsChangedNotification;
 - (void)textViewInvalidateRestorableState;
+- (void)textViewDidFindDirtyRects;
 - (void)textViewBeginDrag;
 - (void)textViewMovePane;
 - (void)textViewSwapPane;
@@ -147,7 +158,7 @@ typedef NS_ENUM(NSInteger, PTYTextViewSelectionExtensionUnit) {
 - (BOOL)textViewCanSelectOutputOfLastCommand;
 - (BOOL)textViewCanSelectCurrentCommand;
 - (NSColor *)textViewCursorGuideColor;
-- (BOOL)textViewUseHFSPlusMapping;
+- (iTermUnicodeNormalization)textViewUnicodeNormalizationForm;
 - (NSColor *)textViewBadgeColor;
 - (NSDictionary *)textViewVariables;
 - (BOOL)textViewSuppressingAllOutput;
@@ -165,11 +176,42 @@ typedef NS_ENUM(NSInteger, PTYTextViewSelectionExtensionUnit) {
 // Update the text view's frame needed.
 - (void)textViewResizeFrameIfNeeded;
 
+- (NSInteger)textViewUnicodeVersion;
+- (void)textViewDidRefresh;
+
+// The background color in the color map changed.
+- (void)textViewBackgroundColorDidChange;
+
+// Describes the current user, host, and path.
+- (NSURL *)textViewCurrentLocation;
+- (void)textViewBurySession;
+- (void)textViewShowHoverURL:(NSString *)url;
+
+- (BOOL)textViewCopyMode;
+- (BOOL)textViewCopyModeSelecting;
+- (VT100GridCoord)textViewCopyModeCursorCoord;
+- (BOOL)textViewPasswordInput;
+- (void)textViewDidSelectRangeForFindOnPage:(VT100GridCoordRange)range;
+- (void)textViewNeedsDisplayInRect:(NSRect)rect;
+- (void)textViewDidSelectPasswordPrompt;
+- (NSImage *)textViewBackgroundImage;
+- (BOOL)backgroundImageTiled;
+- (BOOL)textViewShouldDrawRect;
+- (void)textViewDidHighightMark;
+
+@end
+
+@interface iTermHighlightedRow : NSObject
+@property (nonatomic, readonly) long long absoluteLineNumber;
+@property (nonatomic, readonly) NSTimeInterval creationDate;
+@property (nonatomic, readonly) BOOL success;
 @end
 
 @interface PTYTextView : NSView <
   iTermColorMapDelegate,
+  iTermIndicatorsHelperDelegate,
   iTermSemanticHistoryControllerDelegate,
+  iTermTextDrawingHelperDelegate,
   NSDraggingDestination,
   NSTextInputClient,
   PointerControllerDelegate>
@@ -202,6 +244,10 @@ typedef NS_ENUM(NSInteger, PTYTextViewSelectionExtensionUnit) {
 // Draw text with light font smoothing?
 @property(nonatomic, assign) iTermThinStrokesSetting thinStrokes;
 
+// Are ligatures allowed?
+@property(nonatomic, assign) BOOL asciiLigatures;
+@property(nonatomic, assign) BOOL nonAsciiLigatures;
+
 // Use a bright version of the text color for bold text?
 @property(nonatomic, assign) BOOL useBrightBold;
 
@@ -235,13 +281,13 @@ typedef NS_ENUM(NSInteger, PTYTextViewSelectionExtensionUnit) {
 // Returns the entire content of the view as a string.
 @property(nonatomic, readonly) NSString *content;
 
-// Returns the time (since 1970) when the selection was last modified, or 0 if there is no selection
-@property(nonatomic, readonly) NSTimeInterval selectionTime;
-
 // Regular and non-ascii fonts.
 @property(nonatomic, readonly) NSFont *font;
 @property(nonatomic, readonly) NSFont *nonAsciiFont;
 
+@property(nonatomic, readonly) PTYFontInfo *primaryFont;
+@property(nonatomic, readonly) PTYFontInfo *secondaryFont;  // non-ascii font, only used if self.useNonAsciiFont is set.
+
 // Returns the non-ascii font, even if it's not being used.
 @property(nonatomic, readonly) NSFont *nonAsciiFontEvenIfNotUsed;
 
@@ -249,6 +295,9 @@ typedef NS_ENUM(NSInteger, PTYTextViewSelectionExtensionUnit) {
 @property(nonatomic, readonly) double lineHeight;
 @property(nonatomic, readonly) double charWidth;
 
+@property(nonatomic, readonly) double charWidthWithoutSpacing;
+@property(nonatomic, readonly) double charHeightWithoutSpacing;
+
 // Is the cursor visible? Defaults to YES.
 @property(nonatomic, assign) BOOL cursorVisible;
 
@@ -291,6 +340,27 @@ typedef void (^PTYTextViewDrawingHookBlock)(iTermTextDrawingHelper *);
 // Returns the desired height of this view that exactly fits its contents.
 @property(nonatomic, readonly) CGFloat desiredHeight;
 
+// Lines that are currently visible on the screen.
+@property(nonatomic, readonly) VT100GridRange rangeOfVisibleLines;
+
+// Helps drawing text and background.
+@property (nonatomic, readonly) iTermTextDrawingHelper *drawingHelper;
+
+@property (nonatomic, readonly) double transparencyAlpha;
+
+// Number of times -stealKeyFocus has been called since the last time it
+// was released with releaseKeyFocus.
+@property (nonatomic, readonly) int keyFocusStolenCount;
+
+// Is the cursor eligible to blink?
+@property (nonatomic, readonly) BOOL isCursorBlinking;
+
+@property (nonatomic, readonly) iTermIndicatorsHelper *indicatorsHelper;
+
+@property (nonatomic, readonly) NSArray<iTermHighlightedRow *> *highlightedRows;
+
+@property (nonatomic) BOOL suppressDrawing;
+
 // Returns the size of a cell for a given font. hspace and vspace are multipliers and the width
 // and height.
 + (NSSize)charSizeForFont:(NSFont*)aFont
@@ -395,12 +465,14 @@ typedef void (^PTYTextViewDrawingHookBlock)(iTermTextDrawingHelper *);
 // Begins a new search. You may need to call continueFind repeatedly after this.
 - (void)findString:(NSString*)aString
   forwardDirection:(BOOL)direction
-      ignoringCase:(BOOL)ignoreCase
-             regex:(BOOL)regex
+      mode:(iTermFindMode)mode
         withOffset:(int)offset;
 
 // Remove highlighted terms from previous search.
-- (void)clearHighlights;
+// If resetContext is set then the search state will get reset to empty.
+// Otherwise, search results and highlights are removed and can be updated
+// on the next search.
+- (void)clearHighlights:(BOOL)resetContext;
 
 // Performs a find on the next chunk of text.
 - (BOOL)continueFind:(double *)progress;
@@ -453,7 +525,7 @@ typedef void (^PTYTextViewDrawingHookBlock)(iTermTextDrawingHelper *);
                          suffix:(NSString *)suffix;
 
 - (PTYFontInfo*)getFontForChar:(UniChar)ch
-                     isComplex:(BOOL)complex
+                     isComplex:(BOOL)isComplex
                     renderBold:(BOOL*)renderBold
                   renderItalic:(BOOL*)renderItalic;
 
@@ -491,6 +563,17 @@ typedef void (^PTYTextViewDrawingHookBlock)(iTermTextDrawingHelper *);
                   inDirection:(PTYTextViewSelectionExtensionDirection)direction
                            by:(PTYTextViewSelectionExtensionUnit)unit;
 
+- (void)moveSelectionEndpoint:(PTYTextViewSelectionEndpoint)endpoint
+                  inDirection:(PTYTextViewSelectionExtensionDirection)direction
+                           by:(PTYTextViewSelectionExtensionUnit)unit
+                  cursorCoord:(VT100GridCoord)cursorCoord;
+
+- (VT100GridWindowedRange)rangeByExtendingRange:(VT100GridWindowedRange)existingRange
+                                       endpoint:(PTYTextViewSelectionEndpoint)endpoint
+                                      direction:(PTYTextViewSelectionExtensionDirection)direction
+                                      extractor:(iTermTextExtractor *)extractor
+                                           unit:(PTYTextViewSelectionExtensionUnit)unit;
+
 // For focus follows mouse. Allows a new split pane to become focused even though the mouse pointer
 // is elsewhere. Records the mouse position. Refuses first responder as long as the mouse doesn't
 // move.
@@ -501,6 +584,10 @@ typedef void (^PTYTextViewDrawingHookBlock)(iTermTextDrawingHelper *);
 
 - (void)setTransparencyAffectsOnlyDefaultBackgroundColor:(BOOL)value;
 
+- (void)showFireworks;
+
+- (BOOL)imageIsVisible:(iTermImageInfo *)image;
+
 #pragma mark - Testing only
 
 - (id)selectedTextAttributed:(BOOL)attributed
