@@ -26,6 +26,7 @@
 
 #import "DebugLogging.h"
 #import "ITAddressBookMgr.h"
+#import "NSMutableAttributedString+iTerm.h"
 #import "PTYSession.h"
 #import "ProfileModel.h"
 #import "ProfileModelWrapper.h"
@@ -64,16 +65,27 @@ const CGFloat kDefaultTagsWidth = 80;
     ProfileTagsView *tagsView_;
     NSSplitView *splitView_;
     CGFloat lastTagsWidth_;
+    NSMutableDictionary<NSNumber *, NSNumber *> *_savedHeights;
+
+    // Cached row height info
+    BOOL _haveHeights;
+    CGFloat _heightWithTags;
+    CGFloat _heightWithoutTags;
 }
 
 - (instancetype)initWithFrame:(NSRect)frameRect {
     return [self initWithFrame:frameRect model:[ProfileModel sharedInstance]];
 }
 
-// This is the designated initializer.
+
 - (instancetype)initWithFrame:(NSRect)frameRect model:(ProfileModel*)dataSource {
+    return [self initWithFrame:frameRect model:dataSource font:nil];
+}
+
+- (instancetype)initWithFrame:(NSRect)frameRect model:(ProfileModel *)dataSource font:(NSFont *)font {
     self = [super initWithFrame:frameRect];
     if (self) {
+        _savedHeights = [[NSMutableDictionary alloc] init];
         margin_ = kInterWidgetMargin;
         [self setUnderlyingDatasource:dataSource];
         debug = NO;
@@ -86,7 +98,9 @@ const CGFloat kDefaultTagsWidth = 80;
         searchFieldFrame.size.width = frame.size.width;
         searchField_ = [[iTermSearchField alloc] initWithFrame:searchFieldFrame];
         [self _addTags:[[dataSource_ underlyingModel] allTags] toSearchField:searchField_];
+        ITERM_IGNORE_PARTIAL_BEGIN
         [searchField_ setDelegate:self];
+        ITERM_IGNORE_PARTIAL_END
         [self addSubview:searchField_];
         self.delegate = nil;
 
@@ -143,8 +157,6 @@ const CGFloat kDefaultTagsWidth = 80;
         [scrollView_ setDocumentView:tableView_];
         [scrollView_ setBorderType:NSBezelBorder];
 
-        [tableView_ setDelegate:self];
-        [tableView_ setDataSource:self];
         selectedGuids_ = [[NSMutableSet alloc] init];
 
         [tableView_ setDoubleAction:@selector(onDoubleClick:)];
@@ -174,15 +186,29 @@ const CGFloat kDefaultTagsWidth = 80;
         tagsView_.delegate = self;
         [splitView_ addSubview:tagsView_];
         [splitView_ addSubview:scrollView_];
+
+        if (font) {
+            [self setFont:font];
+        }
+        [tableView_ setDataSource:self];
+        [tableView_ setDelegate:self];
     }
     return self;
 }
 
-- (void)dealloc
-{
+- (void)dealloc {
     [[NSNotificationCenter defaultCenter] removeObserver:self];
+    [_savedHeights release];
     [dataSource_ release];
     [selectedGuids_ release];
+    // These if statements are pure paranoia because this thing gets used all
+    // over the place and is pretty old.
+    if (tableView_.delegate == self) {
+        tableView_.delegate = nil;
+    }
+    if (tableView_.dataSource == self) {
+        tableView_.dataSource = nil;
+    }
     [super dealloc];
 }
 
@@ -461,20 +487,38 @@ const CGFloat kDefaultTagsWidth = 80;
     [self reloadData];
 }
 
-- (CGFloat)tableView:(NSTableView *)tableView heightOfRow:(NSInteger)rowIndex {
-    NSCell *cell = [tableView preparedCellAtColumn:[[tableView tableColumns] indexOfObject:tableColumn_]
-                                               row:rowIndex];
-    NSRect constrainedBounds = NSMakeRect(0, 0, tableColumn_.width, CGFLOAT_MAX);
-    NSSize naturalSize = [cell cellSizeForBounds:constrainedBounds];
-
-    // I have no idea why I need extraHeight but maybe cellSizeForBounds: doesn't center content
-    // properly with attributed strings.
-    return naturalSize.height + [self extraHeight];
+- (CGFloat)heightOfRowWithTags:(BOOL)hasTags {
+    if (!_haveHeights) {
+        _heightWithTags = [[self attributedStringForName:@"M"
+                                                    tags:@[ @"M" ]
+                                                selected:NO
+                                               isDefault:YES
+                                                  filter:nil] heightForWidth:100] + [self extraHeightWithTags:YES];
+        _heightWithoutTags = [[self attributedStringForName:@"M"
+                                                       tags:nil
+                                                   selected:NO
+                                                  isDefault:YES
+                                                     filter:nil] heightForWidth:100] + [self extraHeightWithTags:NO];
+        _haveHeights = YES;
+    }
+    return hasTags ? _heightWithTags : _heightWithoutTags;
 }
 
-- (CGFloat)extraHeight {
+- (CGFloat)tableView:(NSTableView *)tableView heightOfRow:(NSInteger)rowIndex {
+    Profile *profile = [dataSource_ profileAtIndex:rowIndex];
+    const BOOL hasTags = ([profile[KEY_TAGS] count] > 0);
+    CGFloat height = [self heightOfRowWithTags:hasTags];
+    _savedHeights[@(rowIndex)] = @(height);
+    return height;
+}
+
+- (CGFloat)extraHeightWithTags:(BOOL)hasTags {
     if (self.mainFont.pointSize <= [NSFont smallSystemFontSize]) {
-        return 1;
+        if (hasTags) {
+            return 2;
+        } else {
+            return 1;
+        }
     } else {
         return 2;
     }
@@ -515,14 +559,21 @@ const CGFloat kDefaultTagsWidth = 80;
         tagColor = [NSColor colorWithCalibratedWhite:0.5 alpha:1];
         highlightedBackgroundColor = [NSColor colorWithCalibratedRed:1 green:1 blue:0 alpha:0.4];
     }
+    NSMutableParagraphStyle *paragraphStyle = [[[NSMutableParagraphStyle alloc] init] autorelease];
+    paragraphStyle.lineBreakMode = NSLineBreakByTruncatingTail;
+
     NSDictionary* plainAttributes = @{ NSForegroundColorAttributeName: textColor,
+                                       NSParagraphStyleAttributeName: paragraphStyle,
                                        NSFontAttributeName: self.mainFont };
     NSDictionary* highlightedNameAttributes = @{ NSForegroundColorAttributeName: textColor,
+                                                 NSParagraphStyleAttributeName: paragraphStyle,
                                                  NSBackgroundColorAttributeName: highlightedBackgroundColor,
                                                  NSFontAttributeName: self.mainFont };
     NSDictionary* smallAttributes = @{ NSForegroundColorAttributeName: tagColor,
+                                       NSParagraphStyleAttributeName: paragraphStyle,
                                        NSFontAttributeName: self.tagFont };
     NSDictionary* highlightedSmallAttributes = @{ NSForegroundColorAttributeName: tagColor,
+                                                  NSParagraphStyleAttributeName: paragraphStyle,
                                                   NSBackgroundColorAttributeName: highlightedBackgroundColor,
                                                   NSFontAttributeName: self.tagFont };
     NSMutableAttributedString *theAttributedString =
@@ -560,7 +611,10 @@ const CGFloat kDefaultTagsWidth = 80;
 }
 
 - (NSAttributedString *)attributedStringForString:(NSString *)string selected:(BOOL)selected {
+    NSMutableParagraphStyle *paragraphStyle = [[[NSMutableParagraphStyle alloc] init] autorelease];
+    paragraphStyle.lineBreakMode = NSLineBreakByTruncatingTail;
     NSDictionary *attributes = @{ NSFontAttributeName: [NSFont systemFontOfSize:[NSFont systemFontSize]],
+                                  NSParagraphStyleAttributeName: paragraphStyle,
                                   NSForegroundColorAttributeName: (selected && [NSApp isActive] && self.window.isKeyWindow) ? [NSColor whiteColor] : [NSColor blackColor] };
     return [[[NSAttributedString alloc] initWithString:string attributes:attributes] autorelease];
 }
@@ -574,7 +628,7 @@ const CGFloat kDefaultTagsWidth = 80;
         DLog(@"Getting name of profile at row %d. The dictionary's address is %p. Its name is %@",
              (int)rowIndex, bookmark, bookmark[KEY_NAME]);
         Profile *defaultProfile = [[ProfileModel sharedInstance] defaultBookmark];
-        return [self attributedStringForName:bookmark[KEY_NAME]
+        return [self attributedStringForName:bookmark[KEY_NAME] ?: @""
                                         tags:bookmark[KEY_TAGS]
                                     selected:[[tableView_ selectedRowIndexes] containsIndex:rowIndex]
                                    isDefault:[bookmark[KEY_GUID] isEqualToString:defaultProfile[KEY_GUID]]
@@ -780,6 +834,9 @@ const CGFloat kDefaultTagsWidth = 80;
 
 - (BOOL)control:(NSControl *)control textView:(NSTextView *)textView doCommandBySelector:(SEL)commandSelector {
     if (commandSelector == @selector(cancelOperation:)) {
+        if (searchField_.stringValue.length == 0) {
+            return NO;
+        }
         searchField_.stringValue = @"";
         dataSource_.lockedGuid = nil;
         [self updateResultsForSearch];
@@ -851,8 +908,7 @@ const CGFloat kDefaultTagsWidth = 80;
                                                              object:nil]];
 }
 
-- (void)resizeSubviewsWithOldSize:(NSSize)oldBoundsSize
-{
+- (void)resizeSubviewsWithOldSize:(NSSize)oldBoundsSize {
     NSRect frame = [self frame];
 
     NSRect searchFieldFrame;
@@ -867,6 +923,20 @@ const CGFloat kDefaultTagsWidth = 80;
                                        frame.size.width,
                                        frame.size.height - kSearchWidgetHeight - margin_);
     splitView_.frame = splitViewFrame;
+
+    if (tableView_.delegate) {
+        NSMutableIndexSet *rowsWithHeightChange = [NSMutableIndexSet indexSet];
+        for (NSInteger i = 0; i < self.numberOfRows; i++) {
+            CGFloat savedHeight = [_savedHeights[@(i)] doubleValue];
+            CGFloat height = [self tableView:tableView_ heightOfRow:i];
+            if (round(height) != round(savedHeight)) {
+                [rowsWithHeightChange addIndex:i];
+            }
+        }
+        if (rowsWithHeightChange.count > 0) {
+            [tableView_ noteHeightOfRowsWithIndexesChanged:rowsWithHeightChange];
+        }
+    }
 }
 
 - (void)turnOnDebug
@@ -882,6 +952,7 @@ const CGFloat kDefaultTagsWidth = 80;
 
 - (void)setFont:(NSFont *)theFont
 {
+    _haveHeights = NO;
     for (NSTableColumn *col in [tableView_ tableColumns]) {
         [[col dataCell] setFont:theFont];
     }
