@@ -16,6 +16,8 @@
 #import "iTermProfilePreferences.h"
 #import "iTermSizeRememberingView.h"
 #import "iTermWarning.h"
+#import "NSArray+iTerm.h"
+#import "NSDictionary+Profile.h"
 #import "PreferencePanel.h"
 #import "ProfileListView.h"
 #import "ProfilesAdvancedPreferencesViewController.h"
@@ -27,15 +29,29 @@
 #import "ProfilesWindowPreferencesViewController.h"
 #import "ProfilesSessionPreferencesViewController.h"
 
+@interface NSWindow(LazyHacks)
+- (void)safeMakeFirstResponder:(id)view;
+@end
+
+@implementation NSWindow(LazyHacks)
+
+- (void)safeMakeFirstResponder:(NSView *)view {
+    if (view.window == self) {
+        [self makeFirstResponder:view];
+    }
+}
+
+@end
+
 static NSString *const kRefreshProfileTable = @"kRefreshProfileTable";
 static const CGFloat kExtraMarginBetweenWindowBottomAndTabViewForEditCurrentSessionMode = 7;
 static const CGFloat kSideMarginsWithinInnerTabView = 11;
 NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEditing";
+NSString *const kProfileSessionHotkeyDidChange = @"kProfileSessionHotkeyDidChange";
 
 @interface ProfilePreferencesViewController () <
     iTermProfilePreferencesBaseViewControllerDelegate,
     NSTabViewDelegate,
-    NSWindowDelegate,
     ProfileListViewDelegate,
     ProfilesGeneralPreferencesViewControllerDelegate>
 @end
@@ -134,6 +150,10 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
     [super setPreferencePanel:preferencePanel];
 }
 
+- (void)windowWillClose {
+    [_profilesListView unlockSelection];
+}
+
 #pragma mark - NSViewController
 
 - (void)awakeFromNib {
@@ -226,6 +246,7 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
     [_windowViewController layoutSubviewsForEditCurrentSessionMode];
     [_sessionViewController layoutSubviewsForEditCurrentSessionMode];
     [_advancedViewController layoutSubviewsForEditCurrentSessionMode];
+    [_keysViewController layoutSubviewsForEditCurrentSessionMode];
     NSRect newFrame = _tabView.superview.bounds;
     newFrame.size.width -= 13;
 
@@ -263,6 +284,19 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
     [[self tabViewControllers] makeObjectsPerformSelector:@selector(reloadProfile)];
 }
 
+- (void)openToProfileWithGuidAndEditHotKey:(NSString *)guid {
+    [_profilesListView reloadData];
+    if ([[self selectedProfile][KEY_GUID] isEqualToString:guid]) {
+        [self reloadProfileInProfileViewControllers];
+    } else {
+        [self selectGuid:guid];
+    }
+    if (!self.view.window.attachedSheet) {
+        [_tabView selectTabViewItem:_keysTab];
+        [_keysViewController openHotKeyPanel:nil];
+    }
+}
+
 - (void)openToProfileWithGuid:(NSString *)guid selectGeneralTab:(BOOL)selectGeneralTab {
     [_profilesListView reloadData];
     if ([[self selectedProfile][KEY_GUID] isEqualToString:guid]) {
@@ -273,7 +307,9 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
     if (selectGeneralTab && !self.view.window.attachedSheet) {
         [_tabView selectTabViewItem:_generalTab];
     }
-    [self.view.window performSelector:@selector(makeFirstResponder:)
+    // Use safeMakeFirstResponder in case the profile name is not visible at the time it is called.
+    // Prevents an annoying message threatening a crash that never comes.
+    [self.view.window performSelector:@selector(safeMakeFirstResponder:)
                            withObject:_generalViewController.profileNameFieldForEditCurrentSession
                            afterDelay:0];
 }
@@ -369,7 +405,10 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
 
 - (void)resizeWindowForTabViewItem:(NSTabViewItem *)tabViewItem animated:(BOOL)animated {
     iTermSizeRememberingView *theView = (iTermSizeRememberingView *)[tabViewItem view];
+    [self resizeWindowForView:theView animated:animated];
+}
 
+- (void)resizeWindowForView:(iTermSizeRememberingView *)theView animated:(BOOL)animated {
     // The window's size includes all space around the tab view, plus the tab view.
     // These variables hold the space on each side of the tab view.
     CGFloat spaceAbove = 0;
@@ -536,11 +575,11 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
     bulkCopyController.keysForKeyboard = [_keysViewController keysForBulkCopy];
     bulkCopyController.keysForAdvanced = [_advancedViewController keysForBulkCopy];
 
-    [NSApp beginSheet:bulkCopyController.window
-       modalForWindow:self.view.window
-        modalDelegate:self
-       didEndSelector:@selector(bulkCopyControllerCloseSheet:returnCode:contextInfo:)
-          contextInfo:bulkCopyController];
+    [self.view.window beginSheet:bulkCopyController.window completionHandler:^(NSModalResponse returnCode) {
+        [bulkCopyController.window close];
+        [bulkCopyController autorelease];
+        [[_delegate profilePreferencesModel] flush];
+    }];
 }
 
 - (IBAction)duplicateProfile:(id)sender
@@ -558,7 +597,8 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
     newProfile[KEY_DEFAULT_BOOKMARK] = @"No";
     newProfile[KEY_SHORTCUT] = @"";
     newProfile[KEY_BOUND_HOSTS] = @[];
-
+    newProfile[KEY_TAGS] = [newProfile[KEY_TAGS] arrayByRemovingObject:kProfileDynamicTag];
+    newProfile[KEY_HAS_HOTKEY] = @NO;  // Don't create another hotkey window with the same hotkey
     [[_delegate profilePreferencesModel] addBookmark:newProfile];
     [_profilesListView reloadData];
     [_profilesListView selectRowByGuid:newProfile[KEY_GUID]];
@@ -614,11 +654,10 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
     [pasteboard writeObjects:@[ profiles ]];
 
     if (errors) {
-        [NSAlert alertWithMessageText:@"Error"
-                        defaultButton:@"Ok"
-                      alternateButton:nil
-                          otherButton:nil
-            informativeTextWithFormat:@"An error occurred. Check Console.app for details."];
+        NSAlert *alert = [[[NSAlert alloc] init] autorelease];
+        alert.messageText = @"Error";
+        alert.informativeText = @"An error occurred. Check Console.app for details.";
+        [alert runModal];
     }
 }
 
@@ -636,12 +675,11 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
         [pasteboard clearContents];
         [pasteboard writeObjects:@[ string ]];
     } else {
-        [NSAlert alertWithMessageText:@"Error"
-                        defaultButton:@"Ok"
-                      alternateButton:nil
-                          otherButton:nil
-            informativeTextWithFormat:@"Couldn't convert profile to JSON: %@",
-                                      [error localizedDescription]];
+        NSAlert *alert = [[[NSAlert alloc] init] autorelease];
+        alert.messageText = @"Error";
+        alert.informativeText = [NSString stringWithFormat:@"Couldn't convert profile to JSON: %@",
+                                 [error localizedDescription]];
+        [alert runModal];
     }
 }
 
@@ -661,16 +699,6 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
     }
 }
 
-#pragma mark - Sheet
-
-- (void)bulkCopyControllerCloseSheet:(NSWindow *)sheet
-                          returnCode:(int)returnCode
-                         contextInfo:(BulkCopyProfilePreferencesWindowController *)bulkCopyController {
-    [sheet close];
-    [bulkCopyController autorelease];
-    [[_delegate profilePreferencesModel] flush];
-}
-
 #pragma mark - iTermProfilesPreferencesBaseViewControllerDelegate
 
 - (Profile *)profilePreferencesCurrentProfile {
@@ -681,6 +709,12 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
     return [_delegate profilePreferencesModel];
 }
 
+- (void)profilePreferencesContentViewSizeDidChange:(iTermSizeRememberingView *)view {
+    if (_tabView.selectedTabViewItem.view == view ){
+        [self resizeWindowForView:view animated:YES];
+    }
+}
+
 
 #pragma mark - ProfilesGeneralPreferencesViewControllerDelegate
 
@@ -697,17 +731,15 @@ NSString *const kProfileSessionNameDidEndEditing = @"kProfileSessionNameDidEndEd
                                                         object:[_profilesListView selectedGuid]];
 }
 
+- (void)profilesGeneralPreferencesSessionHotkeyDidChange {
+    [[NSNotificationCenter defaultCenter] postNotificationName:kProfileSessionHotkeyDidChange
+                                                        object:[_profilesListView selectedGuid]];
+}
+
 #pragma mark - NSTabViewDelegate
 
 - (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem {
     [self resizeWindowForTabViewItem:tabViewItem animated:YES];
 }
 
-#pragma mark - NSWindowDelegate
-
-- (void)windowWillClose:(NSNotification *)notification {
-    [_generalViewController windowWillClose];
-    [_profilesListView unlockSelection];
-}
-
 @end
