@@ -16,7 +16,7 @@
 #import "iTermWarning.h"
 #import "NSFileManager+iTerm.h"
 #import "NSObject+iTerm.h"
-#import "NSStringiTerm.h"
+#import "NSStringITerm.h"
 
 @interface NMSSHSession(iTerm)
 - (id)agent;
@@ -190,7 +190,7 @@ static NSError *SCPFileError(NSString *description) {
 - (NSArray *)configs {
     NSFileManager *fileManager = [NSFileManager defaultManager];
     NSString *appSupport = [fileManager applicationSupportDirectory];
-    NSArray *paths = @[ [appSupport stringByAppendingPathExtension:@"iTerm/ssh_config"] ?: @"",
+    NSArray *paths = @[ [appSupport stringByAppendingPathComponent:@"ssh_config"] ?: @"",
                         [@"~/.ssh/config" stringByExpandingTildeInPath] ?: @"",
                         @"/etc/ssh/ssh_config",
                         @"/etc/ssh_config" ];
@@ -205,7 +205,7 @@ static NSError *SCPFileError(NSString *description) {
             if (config) {
                 [configs addObject:config];
             } else {
-                ELog(@"Could not parse config file at %@", path);
+                XLog(@"Could not parse config file at %@", path);
             }
         }
     }
@@ -244,17 +244,20 @@ static NSError *SCPFileError(NSString *description) {
         return;
     }
     _okToAdd = NO;
+    int effectivePort;
     if (self.session) {
         self.session.delegate = self;
+        effectivePort = self.session.port.intValue;
     } else {
         self.session = [[[NMSSHSession alloc] initWithHost:[self hostname]
                                                    configs:[self configs]
                                            withDefaultPort:[self port]
                                            defaultUsername:self.path.username] autorelease];
+        effectivePort = self.session.port.intValue;
         self.session.delegate = self;
         [self.session connect];
         if (self.stopped) {
-            ELog(@"Stop after connect");
+            XLog(@"Stop after connect");
             dispatch_sync(dispatch_get_main_queue(), ^() {
                 [[FileTransferManager sharedInstance] transferrableFileDidStopTransfer:self];
             });
@@ -276,7 +279,7 @@ static NSError *SCPFileError(NSString *description) {
             [[FileTransferManager sharedInstance] transferrableFile:self
                                      didFinishTransmissionWithError:theError];
             iTermWarningSelection selection =
-                [iTermWarning showWarningWithTitle:[NSString stringWithFormat:@"Failed to connect to %@:%d. Double-check that the host name is correct.", self.hostname, self.port]
+                [iTermWarning showWarningWithTitle:[NSString stringWithFormat:@"Failed to connect to %@:%d. Double-check that the host name is correct.", self.hostname, effectivePort]
                                            actions:@[ @"Ok", @"Help" ]
                                      actionMapping:nil
                                          accessory:nil
@@ -307,11 +310,11 @@ static NSError *SCPFileError(NSString *description) {
         }
         for (NSString *authType in authTypes) {
             if (self.stopped) {
-                ELog(@"Break out of auth loop because stopped");
+                XLog(@"Break out of auth loop because stopped");
                 break;
             }
             if (!self.session.session) {
-                ELog(@"Break out of auth loop because disconnected");
+                XLog(@"Break out of auth loop because disconnected");
                 break;
             }
             if ([authType isEqualToString:@"password"]) {
@@ -356,7 +359,7 @@ static NSError *SCPFileError(NSString *description) {
                 for (NSString *keyPath in keyPaths) {
                     keyPath = [self filenameByExpandingMetasyntaticVariables:keyPath];
                     if (![fileManager fileExistsAtPath:keyPath]) {
-                        ELog(@"No key file at %@", keyPath);
+                        XLog(@"No key file at %@", keyPath);
                         continue;
                     }
                     __block NSString *password = nil;
@@ -369,10 +372,10 @@ static NSError *SCPFileError(NSString *description) {
                                                                      keyboardInteractivePrompt:prompt];
                         });
                     }
-                    ELog(@"Attempting to authenticate with key %@", keyPath);
+                    XLog(@"Attempting to authenticate with key %@", keyPath);
                     NSString *publicKeyPath = [keyPath stringByAppendingString:@".pub"];
                     if (![[NSFileManager defaultManager] fileExistsAtPath:publicKeyPath]) {
-                        ELog(@"Warning: no public key at %@. Trying to authenticate with only a private key.", publicKeyPath);
+                        XLog(@"Warning: no public key at %@. Trying to authenticate with only a private key.", publicKeyPath);
                         publicKeyPath = nil;
                     }
                     [self.session authenticateByPublicKey:publicKeyPath
@@ -380,12 +383,12 @@ static NSError *SCPFileError(NSString *description) {
                                               andPassword:password];
                 
                     if (self.session.isAuthorized) {
-                        ELog(@"Authorized!");
+                        XLog(@"Authorized!");
                         break;
                     }
 
                     if (!self.session.session) {
-                        ELog(@"Disconnected!");
+                        XLog(@"Disconnected!");
                         break;
                     }
                 }
@@ -396,7 +399,7 @@ static NSError *SCPFileError(NSString *description) {
         }
     }
     if (self.stopped) {
-        ELog(@"Stop after auth");
+        XLog(@"Stop after auth");
         dispatch_sync(dispatch_get_main_queue(), ^() {
             [[FileTransferManager sharedInstance] transferrableFileDidStopTransfer:self];
         });
@@ -460,7 +463,7 @@ static NSError *SCPFileError(NSString *description) {
                                                     }
                                                 });
                                                 if (self.stopped) {
-                                                    ELog(@"Stopping mid-download");
+                                                    XLog(@"Stopping mid-download");
                                                 }
                                                 return !self.stopped;
                                             }];
@@ -639,7 +642,7 @@ static NSError *SCPFileError(NSString *description) {
                 title = [NSString stringWithFormat:@"First time connecting to %@", session.host];
                 message =
                     [NSString stringWithFormat:@"The authenticity of host '%@' can't be established.\n\n"
-                                               @"DSA key fingerprint is %@.\n\nConnect anyay?",
+                                               @"DSA key fingerprint is %@.\n\nConnect anyway?",
                         session.host, fingerprint];
                 _okToAdd = YES;
                 break;
