@@ -29,9 +29,12 @@
 
 #import "DebugLogging.h"
 #import "FutureMethods.h"
-#import "HotkeyWindowController.h"
 #import "ITAddressBookMgr.h"
 #import "iTermAdvancedSettingsModel.h"
+#import "iTermApplication.h"
+#import "iTermBuriedSessions.h"
+#import "iTermHotKeyController.h"
+#import "NSArray+iTerm.h"
 #import "NSFileManager+iTerm.h"
 #import "NSStringITerm.h"
 #import "NSURL+iTerm.h"
@@ -58,6 +61,7 @@
 #import "iTermRestorableSession.h"
 #import "iTermSystemVersion.h"
 #import "iTermWarning.h"
+#import "PTYWindow.h"
 #include <objc/runtime.h>
 
 @interface NSApplication (Undocumented)
@@ -151,7 +155,7 @@ static iTermController *gSharedInstance;
     const BOOL sessionsWillRestore = ([iTermAdvancedSettingsModel runJobsInServers] &&
                                       [iTermAdvancedSettingsModel restoreWindowContents] &&
                                       self.willRestoreWindowsAtNextLaunch);
-    iTermApplicationDelegate *itad = iTermApplication.sharedApplication.delegate;
+    iTermApplicationDelegate *itad = [iTermApplication.sharedApplication delegate];
     return (sessionsWillRestore &&
             (itad.sparkleRestarting || ![iTermAdvancedSettingsModel killJobsInServersOnQuit]));
 }
@@ -160,7 +164,7 @@ static iTermController *gSharedInstance;
     [[NSNotificationCenter defaultCenter] removeObserver:self];
 
     // Save hotkey window arrangement to user defaults before closing it.
-    [[HotkeyWindowController sharedInstance] saveHotkeyWindowState];
+    [[iTermHotKeyController sharedInstance] saveHotkeyWindowStates];
     [[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:self];
 
     if (self.shouldLeaveSessionsRunningOnQuit) {
@@ -187,6 +191,7 @@ static iTermController *gSharedInstance;
     [_restorableSessions release];
     [_currentRestorableSessionsStack release];
     [_fullScreenWindowManager release];
+    [_lastSelection release];
     [super dealloc];
 }
 
@@ -219,6 +224,11 @@ static iTermController *gSharedInstance;
             }
         }
     }
+    for (PTYSession *session in [[iTermBuriedSessions sharedInstance] buriedSessions]) {
+        if ([session isTmuxClient] || [session isTmuxGateway]) {
+            return session;
+        }
+    }
     return nil;
 }
 
@@ -249,21 +259,15 @@ static iTermController *gSharedInstance;
 - (BOOL)terminalIsObscured:(id<iTermWindowController>)terminal {
     BOOL windowIsObscured = NO;
     NSWindow *window = [terminal window];
-    // occlusionState is new in 10.9.
-    if ([window respondsToSelector:@selector(occlusionState)]) {
-        NSWindowOcclusionState occlusionState = window.occlusionState;
-        // The occlusionState tells if you if you're on another space or another app's window is
-        // occluding yours, but for some reason one terminal window can occlude another without
-        // it noticing, so we compute that ourselves.
-        windowIsObscured = !(occlusionState & NSWindowOcclusionStateVisible);
-    } else {
-        // Use a very rough approximation. Users who complain should upgrade to 10.9.
-        windowIsObscured = !window.isOnActiveSpace;
-    }
+    NSWindowOcclusionState occlusionState = window.occlusionState;
+    // The occlusionState tells if you if you're on another space or another app's window is
+    // occluding yours, but for some reason one terminal window can occlude another without
+    // it noticing, so we compute that ourselves.
+    windowIsObscured = !(occlusionState & NSWindowOcclusionStateVisible);
     if (!windowIsObscured) {
         // Try to refine the guess by seeing if another terminal is covering this one.
         static const double kOcclusionThreshold = 0.4;
-        if ([(PTYWindow *)terminal.window approximateFractionOccluded] > kOcclusionThreshold) {
+        if ([(iTermTerminalWindow *)terminal.window approximateFractionOccluded] > kOcclusionThreshold) {
             windowIsObscured = YES;
         }
     }
@@ -341,50 +345,33 @@ static iTermController *gSharedInstance;
     }
 }
 
-- (NSString *)_showAlertWithText:(NSString *)prompt defaultInput:(NSString *)defaultValue {
-    NSAlert *alert = [NSAlert alertWithMessageText:prompt
-                                     defaultButton:@"OK"
-                                   alternateButton:@"Cancel"
-                                       otherButton:nil
-                         informativeTextWithFormat:@""];
-
-    NSTextField *input = [[[NSTextField alloc] initWithFrame:NSMakeRect(0, 0, 200, 24)] autorelease];
-    [input setStringValue:defaultValue];
-    [alert setAccessoryView:input];
-    [alert layout];
-    [[alert window] makeFirstResponder:input];
-    NSInteger button = [alert runModal];
-    if (button == NSAlertDefaultReturn) {
-        [input validateEditing];
-        return [input stringValue];
-    } else if (button == NSAlertAlternateReturn) {
-        return nil;
-    } else {
-        NSAssert1(NO, @"Invalid input dialog button %d", (int) button);
-        return nil;
+- (void)repairSavedArrangementNamed:(NSString *)savedArrangementName
+               replacingMissingGUID:(NSString *)guidToReplace
+                           withGUID:(NSString *)replacementGuid {
+    NSArray *terminalArrangements = [WindowArrangements arrangementWithName:savedArrangementName];
+    Profile *goodProfile = [[ProfileModel sharedInstance] bookmarkWithGuid:replacementGuid];
+    if (goodProfile) {
+        NSMutableArray *repairedArrangements = [NSMutableArray array];
+        for (NSDictionary *terminalArrangement in terminalArrangements) {
+            [repairedArrangements addObject:[PseudoTerminal repairedArrangement:terminalArrangement
+                                                       replacingProfileWithGUID:guidToReplace
+                                                                    withProfile:goodProfile]];
+        }
+        [WindowArrangements setArrangement:repairedArrangements withName:savedArrangementName];
     }
 }
 
 - (void)saveWindowArrangement:(BOOL)allWindows {
-    NSString *name = [self _showAlertWithText:@"Name for saved window arrangement:"
-                                 defaultInput:[NSString stringWithFormat:@"Arrangement %d", 1 + [WindowArrangements count]]];
+    NSString *name = [WindowArrangements nameForNewArrangement];
     if (!name) {
         return;
     }
-    if ([WindowArrangements hasWindowArrangement:name]) {
-        if (NSRunAlertPanel(@"Replace Existing Saved Window Arrangement?",
-                            @"There is an existing saved window arrangement with this name. Would you like to replace it with the current arrangement?",
-                            @"Yes",
-                            @"No",
-                            nil) != NSAlertDefaultReturn) {
-            return;
-        }
-    }
     NSMutableArray *terminalArrangements = [NSMutableArray arrayWithCapacity:[_terminalWindows count]];
     if (allWindows) {
         for (PseudoTerminal *terminal in _terminalWindows) {
-            if (![terminal isHotKeyWindow]) {
-                [terminalArrangements addObject:[terminal arrangement]];
+            NSDictionary *arrangement = [terminal arrangement];
+            if (arrangement) {
+                [terminalArrangements addObject:arrangement];
             }
         }
     } else {
@@ -392,12 +379,28 @@ static iTermController *gSharedInstance;
         if (!currentTerminal) {
             return;
         }
-        [terminalArrangements addObject:[currentTerminal arrangement]];
+        NSDictionary *arrangement = [currentTerminal arrangement];
+        if (arrangement) {
+            [terminalArrangements addObject:arrangement];
+        }
+    }
+    if (terminalArrangements.count) {
+        [WindowArrangements setArrangement:terminalArrangements withName:name];
     }
-    [WindowArrangements setArrangement:terminalArrangements withName:name];
 }
 
 - (void)tryOpenArrangement:(NSDictionary *)terminalArrangement {
+    [self tryOpenArrangement:terminalArrangement asTabs:NO];
+}
+
+- (void)tryOpenArrangement:(NSDictionary *)terminalArrangement asTabs:(BOOL)asTabs {
+    if (asTabs) {
+        PseudoTerminal *term = [self currentTerminal];
+        if (term) {
+            [term restoreTabsFromArrangement:terminalArrangement sessions:nil];
+            return;
+        }
+    }
     BOOL shouldDelay = NO;
     DLog(@"Try to open arrangement %p...", terminalArrangement);
     if ([PseudoTerminal willAutoFullScreenNewWindow] &&
@@ -415,20 +418,27 @@ static iTermController *gSharedInstance;
         [self performSelector:_cmd withObject:terminalArrangement afterDelay:0.25];
     } else {
         DLog(@"Opening it.");
-        PseudoTerminal* term = [PseudoTerminal terminalWithArrangement:terminalArrangement];
+        PseudoTerminal *term = [PseudoTerminal terminalWithArrangement:terminalArrangement
+                                              forceOpeningHotKeyWindow:NO];
         if (term) {
           [self addTerminalWindow:term];
         }
     }
 }
 
-- (void)loadWindowArrangementWithName:(NSString *)theName {
+- (void)loadWindowArrangementWithName:(NSString *)theName asTabs:(BOOL)asTabs {
+    _savedArrangementNameBeingRestored = [[theName retain] autorelease];
     NSArray *terminalArrangements = [WindowArrangements arrangementWithName:theName];
     if (terminalArrangements) {
         for (NSDictionary *terminalArrangement in terminalArrangements) {
-            [self tryOpenArrangement:terminalArrangement];
+            [self tryOpenArrangement:terminalArrangement asTabs:asTabs];
         }
     }
+    _savedArrangementNameBeingRestored = nil;
+}
+
+- (void)loadWindowArrangementWithName:(NSString *)theName {
+    [self loadWindowArrangementWithName:theName asTabs:NO];
 }
 
 // Return all the terminals in the given screen.
@@ -602,22 +612,6 @@ static iTermController *gSharedInstance;
     }
 }
 
-- (PTYSession *)sessionWithMostRecentSelection {
-    NSTimeInterval latest = 0;
-    PTYSession *best = nil;
-    for (PseudoTerminal *term in [self terminals]) {
-        PTYTab *aTab = [term currentTab];
-        for (PTYSession *aSession in [aTab sessions]) {
-            NSTimeInterval current = [[aSession textview] selectionTime];
-            if (current > latest) {
-                latest = current;
-                best = aSession;
-            }
-        }
-    }
-    return best;
-}
-
 - (PseudoTerminal *)currentTerminal {
     return _frontTerminalWindowController;
 }
@@ -886,8 +880,8 @@ static iTermController *gSharedInstance;
 }
 
 + (void)switchToSpaceInBookmark:(Profile *)aDict {
-    if ([aDict objectForKey:KEY_SPACE]) {
-        int spaceNum = [[aDict objectForKey:KEY_SPACE] intValue];
+    if (aDict[KEY_SPACE]) {
+        int spaceNum = [aDict[KEY_SPACE] intValue];
         if (spaceNum > 0 && spaceNum < 10) {
             // keycodes for digits 1-9. Send control-n to switch spaces.
             // TODO: This would get remapped by the event tap. It requires universal access to be on and
@@ -907,11 +901,13 @@ static iTermController *gSharedInstance;
             // subsequent to this will appear in the previous space. This is short enough of a
             // delay that it's not annoying when you're already there.
             [NSThread sleepForTimeInterval:0.3];
+
+            [NSApp activateIgnoringOtherApps:YES];
         }
     }
 }
 
-- (int)windowTypeForBookmark:(Profile *)aDict {
+- (iTermWindowType)windowTypeForBookmark:(Profile *)aDict {
     if ([aDict objectForKey:KEY_WINDOW_TYPE]) {
         int windowType = [[aDict objectForKey:KEY_WINDOW_TYPE] intValue];
         if (windowType == WINDOW_TYPE_TRADITIONAL_FULL_SCREEN &&
@@ -958,14 +954,24 @@ static iTermController *gSharedInstance;
                                           windowType:windowType
                                      savedWindowType:WINDOW_TYPE_NORMAL
                                               screen:[iTermProfilePreferences intForKey:KEY_SCREEN inProfile:profile]
-                                            isHotkey:NO] autorelease];
+                                    hotkeyWindowType:iTermHotkeyWindowTypeNone] autorelease];
     if ([iTermProfilePreferences boolForKey:KEY_HIDE_AFTER_OPENING inProfile:profile]) {
         [term hideAfterOpening];
     }
+    iTermProfileHotKey *profileHotKey =
+        [[iTermHotKeyController sharedInstance] didCreateWindowController:term
+                                                              withProfile:profile
+                                                                     show:NO];
+    [profileHotKey setAllowsStateRestoration:NO];
+    
     [self addTerminalWindow:term];
     return term;
 }
 
+- (void)didFinishCreatingTmuxWindow:(PseudoTerminal *)windowController {
+    [[[iTermHotKeyController sharedInstance] profileHotKeyForWindowController:windowController] showHotKeyWindow];
+}
+
 - (void)makeTerminalWindowFullScreen:(NSWindowController<iTermWindowController> *)term {
     [[iTermFullScreenWindowManager sharedInstance] makeWindowEnterFullScreen:term.ptyWindow];
 }
@@ -974,7 +980,7 @@ static iTermController *gSharedInstance;
     return [self launchBookmark:bookmarkData
                      inTerminal:theTerm
                         withURL:nil
-                       isHotkey:NO
+                       hotkeyWindowType:iTermHotkeyWindowTypeNone
                         makeKey:YES
                     canActivate:YES
                         command:nil
@@ -999,14 +1005,31 @@ static iTermController *gSharedInstance;
 
         if ([urlType compare:@"ssh" options:NSCaseInsensitiveSearch] == NSOrderedSame) {
             NSMutableString *tempString = [NSMutableString stringWithString:@"ssh "];
-            if ([urlRep user]) {
-                [tempString appendFormat:@"-l %@ ", [urlRep user]];
+            NSString *username = [urlRep user];
+            if (username) {
+                NSMutableCharacterSet *legalCharacters = [NSMutableCharacterSet alphanumericCharacterSet];
+                [legalCharacters addCharactersInString:@"_-+"];
+                NSCharacterSet *illegalCharacters = [legalCharacters invertedSet];
+                NSRange range = [username rangeOfCharacterFromSet:illegalCharacters];
+                if (range.location != NSNotFound) {
+                    ELog(@"username %@ contains illegal character at position %@", username, @(range.location));
+                    return nil;
+                }
+                [tempString appendFormat:@"-l %@ ", [[urlRep user] stringWithEscapedShellCharactersIncludingNewlines:YES]];
             }
             if ([urlRep port]) {
                 [tempString appendFormat:@"-p %@ ", [urlRep port]];
             }
-            if ([urlRep host]) {
-                [tempString appendString:[urlRep host]];
+            NSString *hostname = [urlRep host];
+            if (hostname) {
+                NSCharacterSet *legalCharacters = [NSCharacterSet characterSetWithCharactersInString:@":abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-."];
+                NSCharacterSet *illegalCharacters = [legalCharacters invertedSet];
+                NSRange range = [hostname rangeOfCharacterFromSet:illegalCharacters];
+                if (range.location != NSNotFound) {
+                    ELog(@"Hostname %@ contains illegal character at position %@", hostname, @(range.location));
+                    return nil;
+                }
+                [tempString appendString:[hostname stringWithEscapedShellCharactersIncludingNewlines:YES]];
             }
             [tempDict setObject:tempString forKey:KEY_COMMAND_LINE];
             [tempDict setObject:@"Yes" forKey:KEY_CUSTOM_COMMAND];
@@ -1019,10 +1042,10 @@ static iTermController *gSharedInstance;
         } else if ([urlType compare:@"telnet" options:NSCaseInsensitiveSearch] == NSOrderedSame) {
             NSMutableString *tempString = [NSMutableString stringWithString:@"telnet "];
             if ([urlRep user]) {
-                [tempString appendFormat:@"-l %@ ", [urlRep user]];
+                [tempString appendFormat:@"-l %@ ", [[urlRep user] stringWithEscapedShellCharactersIncludingNewlines:YES]];
             }
             if ([urlRep host]) {
-                [tempString appendString:[urlRep host]];
+                [tempString appendString:[[urlRep host] stringWithEscapedShellCharactersIncludingNewlines:YES]];
                 if ([urlRep port]) [tempString appendFormat:@" %@", [urlRep port]];
             }
             [tempDict setObject:tempString forKey:KEY_COMMAND_LINE];
@@ -1040,15 +1063,15 @@ static iTermController *gSharedInstance;
 - (PTYSession *)launchBookmark:(NSDictionary *)bookmarkData
                     inTerminal:(PseudoTerminal *)theTerm
                        withURL:(NSString *)url
-                      isHotkey:(BOOL)isHotkey
+              hotkeyWindowType:(iTermHotkeyWindowType)hotkeyWindowType
                        makeKey:(BOOL)makeKey
                    canActivate:(BOOL)canActivate
                        command:(NSString *)command
-                         block:(PTYSession *(^)(PseudoTerminal *))block {
+                         block:(PTYSession *(^)(Profile *, PseudoTerminal *))block {
     DLog(@"launchBookmark:inTerminal:withUrl:isHotkey:makeKey:canActivate:command:block:");
     DLog(@"Profile:\n%@", bookmarkData);
     DLog(@"URL: %@", url);
-    DLog(@"isHotkey: %@", @(isHotkey));
+    DLog(@"hotkey window type: %@", @(hotkeyWindowType));
     DLog(@"makeKey: %@", @(makeKey));
     DLog(@"canActivate: %@", @(canActivate));
     DLog(@"command: %@", command);
@@ -1056,7 +1079,8 @@ static iTermController *gSharedInstance;
     PseudoTerminal *term;
     NSDictionary *aDict;
     const iTermObjectType objectType = theTerm ? iTermTabObject : iTermWindowObject;
-
+    const BOOL isHotkey = (hotkeyWindowType != iTermHotkeyWindowTypeNone);
+    
     aDict = bookmarkData;
     if (aDict == nil) {
         aDict = [self defaultBookmark];
@@ -1066,6 +1090,10 @@ static iTermController *gSharedInstance;
         DLog(@"Add URL to profile");
         // Automatically fill in ssh command if command is exactly equal to $$ or it's a login shell.
         aDict = [self profile:aDict modifiedToOpenURL:url forObjectType:objectType];
+        if (aDict == nil) {
+            // Bogus hostname detected
+            return nil;
+        }
     }
     if (!bookmarkData) {
         DLog(@"Using profile:\n%@", aDict);
@@ -1089,14 +1117,14 @@ static iTermController *gSharedInstance;
                                            windowType:windowType
                                       savedWindowType:WINDOW_TYPE_NORMAL
                                                screen:[aDict objectForKey:KEY_SCREEN] ? [[aDict objectForKey:KEY_SCREEN] intValue] : -1
-                                             isHotkey:isHotkey];
+                                     hotkeyWindowType:hotkeyWindowType];
         } else {
             DLog(@"Create a new window controller");
             term = [[[PseudoTerminal alloc] initWithSmartLayout:YES
                                                      windowType:windowType
                                                 savedWindowType:WINDOW_TYPE_NORMAL
                                                          screen:[aDict objectForKey:KEY_SCREEN] ? [[aDict objectForKey:KEY_SCREEN] intValue] : -1
-                                                       isHotkey:isHotkey] autorelease];
+                                               hotkeyWindowType:hotkeyWindowType] autorelease];
         }
         if ([[aDict objectForKey:KEY_HIDE_AFTER_OPENING] boolValue]) {
             [term hideAfterOpening];
@@ -1117,7 +1145,7 @@ static iTermController *gSharedInstance;
 
     if (block) {
         DLog(@"Create a session via callback");
-        session = block(term);
+        session = block(aDict, term);
     } else if (url) {
         DLog(@"Creating a new session");
         session = [term createSessionWithProfile:aDict
@@ -1139,10 +1167,20 @@ static iTermController *gSharedInstance;
         // When this function is activated from the dock icon's context menu make sure
         // that the new window is on top of all other apps' windows. For some reason,
         // makeKeyAndOrderFront does nothing.
+        if ([term.window isKindOfClass:[iTermPanel class]]) {
+            canActivate = NO;
+        }
         if (canActivate) {
-            [NSApp activateIgnoringOtherApps:YES];
+            // activateIgnoringApp: happens asynchronously which means doing makeKeyAndOrderFront:
+            // immediately after it won't do what you want. Issue 6397
+            NSWindow *termWindow = [[term window] retain];
+            [[iTermApplication sharedApplication] activateAppWithCompletion:^{
+                [termWindow makeKeyAndOrderFront:nil];
+                [termWindow release];
+            }];
+        } else {
+            [[term window] makeKeyAndOrderFront:nil];
         }
-        [[term window] makeKeyAndOrderFront:nil];
         if (canActivate) {
             [NSApp arrangeInFront:self];
         }
@@ -1185,12 +1223,10 @@ static iTermController *gSharedInstance;
         location = [NSString stringWithFormat:@"The error starts at byte %d of the script.",
                     (int)[range rangeValue].location];
     }
-    NSAlert *alert = [NSAlert alertWithMessageText:@"Error running script"
-                                     defaultButton:@"OK"
-                                   alternateButton:nil
-                                       otherButton:nil
-                         informativeTextWithFormat:@"Script at \"%@\" failed.\n\nThe error was: \"%@\"\n\n%@",
-                      fullPath, errorInfo[NSAppleScriptErrorMessage], location];
+    NSAlert *alert = [[[NSAlert alloc] init] autorelease];
+    alert.messageText = @"Error running script";
+    alert.informativeText = [NSString stringWithFormat:@"Script at \"%@\" failed.\n\nThe error was: \"%@\"\n\n%@",
+                             fullPath, errorInfo[NSAppleScriptErrorMessage], location];
     [alert runModal];
 }
 
@@ -1206,10 +1242,19 @@ static iTermController *gSharedInstance;
     return [_terminalWindows indexOfObject:terminal];
 }
 
--(PseudoTerminal*)terminalAtIndex:(int)i {
+-(PseudoTerminal *)terminalAtIndex:(int)i {
     return [_terminalWindows objectAtIndex:i];
 }
 
+- (PseudoTerminal *)terminalForWindow:(NSWindow *)window {
+    for (PseudoTerminal *term in _terminalWindows) {
+        if (term.window == window) {
+            return term;
+        }
+    }
+    return nil;
+}
+
 - (int)allocateWindowNumber {
     NSMutableSet* numbers = [NSMutableSet setWithCapacity:[self numberOfTerminals]];
     for (PseudoTerminal* term in [self terminals]) {
@@ -1291,7 +1336,9 @@ static iTermController *gSharedInstance;
     iTermRestorableSession *session = self.currentRestorableSession;
     assert(session);
     if (session) {
-        [_restorableSessions addObject:session];
+        if (session.sessions.count > 0) {
+            [_restorableSessions addObject:session];
+        }
         [_currentRestorableSessionsStack removeObjectAtIndex:0];
     }
 }
@@ -1333,15 +1380,18 @@ static iTermController *gSharedInstance;
     assert([iTermAdvancedSettingsModel runJobsInServers]);
     for (iTermRestorableSession *restorableSession in _restorableSessions) {
         for (PTYSession *aSession in restorableSession.sessions) {
-            [aSession.shell sendSignal:SIGHUP];
+            if (aSession.shell.serverPid != -1) {
+                [aSession.shell sendSignal:SIGKILL toServer:YES];
+            }
+            [aSession.shell sendSignal:SIGHUP toServer:YES];
         }
     }
 }
 
 // This exists because I don't trust -[NSApp keyWindow]. I've seen all kinds of weird behavior from it.
-- (BOOL)anyOrderedWindowIsKey {
+- (BOOL)anyVisibleWindowIsKey {
     DLog(@"Searching for key window...");
-    for (NSWindow *window in [NSApp orderedWindows]) {
+    for (NSWindow *window in [(iTermApplication *)NSApp orderedWindowsPlusVisibleHotkeyPanels]) {
         if (window.isKeyWindow) {
             DLog(@"Key ordered window is %@", window);
             return YES;
@@ -1351,13 +1401,6 @@ static iTermController *gSharedInstance;
     return NO;
 }
 
-- (BOOL)keystrokesBeingStolen {
-    // If we're active and have ordered windows but none of them are key then our keystrokes are
-    // being stolen by something else. This is meant to detect when Spotlight is open. It might
-    // also catch other things that act similarly.
-    return [NSApp isActive] && [[NSApp orderedWindows] count] > 0 && ![self anyOrderedWindowIsKey];
-}
-
 // I don't trust -[NSApp mainWindow].
 - (BOOL)anyWindowIsMain {
     for (NSWindow *window in [[NSApplication sharedApplication] windows]) {
@@ -1368,20 +1411,11 @@ static iTermController *gSharedInstance;
     return NO;
 }
 
-- (PseudoTerminal *)hotkeyWindow {
-    for (PseudoTerminal *term in [[iTermController sharedInstance] terminals]) {
-        if (term.isHotKeyWindow) {
-            return term;
-        }
-    }
-    return nil;
-}
-
 // Returns all terminal windows that are key.
-- (NSArray<PTYWindow *> *)keyTerminalWindows {
-    NSMutableArray<PTYWindow *> *temp = [NSMutableArray array];
+- (NSArray<iTermTerminalWindow *> *)keyTerminalWindows {
+    NSMutableArray<iTermTerminalWindow *> *temp = [NSMutableArray array];
     for (PseudoTerminal *term in [[iTermController sharedInstance] terminals]) {
-        PTYWindow *window = [term ptyWindow];
+        iTermTerminalWindow *window = [term ptyWindow];
         if ([window isKeyWindow]) {
             [temp addObject:window];
         }
@@ -1390,8 +1424,8 @@ static iTermController *gSharedInstance;
 }
 
 // accessors for to-many relationships:
-- (NSArray*)terminals {
-    return (_terminalWindows);
+- (NSArray *)terminals {
+    return _terminalWindows;
 }
 
 - (void)setCurrentTerminal:(PseudoTerminal *)thePseudoTerminal {
@@ -1428,6 +1462,45 @@ static iTermController *gSharedInstance;
 
 - (void)workspaceWillPowerOff:(NSNotification *)notification {
     _willPowerOff = YES;
+    if ([iTermAdvancedSettingsModel killSessionsOnLogout] && [iTermAdvancedSettingsModel runJobsInServers]) {
+        [self killRestorableSessions];
+    }
+}
+
+- (NSInteger)numberOfDecodesPending {
+    return [[self.terminals filteredArrayUsingBlock:^BOOL(PseudoTerminal *anObject) {
+        return anObject.restorableStateDecodePending;
+    }] count];
+}
+
+- (void)openSingleUseWindowWithCommand:(NSString *)command {
+    if ([command hasSuffix:@"&"] && command.length > 1) {
+        command = [command substringToIndex:command.length - 1];
+        system(command.UTF8String);
+        return;
+    }
+    NSString *escapedCommand = [command stringWithEscapedShellCharactersIncludingNewlines:YES];
+    command = [NSString stringWithFormat:@"sh -c \"%@\"", escapedCommand];
+    Profile *windowProfile = [self defaultBookmark];
+    if ([windowProfile[KEY_WINDOW_TYPE] integerValue] == WINDOW_TYPE_TRADITIONAL_FULL_SCREEN ||
+        [windowProfile[KEY_WINDOW_TYPE] integerValue] == WINDOW_TYPE_LION_FULL_SCREEN) {
+        windowProfile = [windowProfile dictionaryBySettingObject:@(WINDOW_TYPE_NORMAL) forKey:KEY_WINDOW_TYPE];
+    }
+    [self launchBookmark:windowProfile
+              inTerminal:nil
+                 withURL:nil
+        hotkeyWindowType:iTermHotkeyWindowTypeNone
+                 makeKey:YES
+             canActivate:YES
+                 command:command
+                   block:^PTYSession *(Profile *profile, PseudoTerminal *term) {
+                       profile = [profile dictionaryBySettingObject:@"" forKey:KEY_INITIAL_TEXT];
+                       profile = [profile dictionaryBySettingObject:@YES forKey:KEY_CLOSE_SESSIONS_ON_END];
+                       term.window.collectionBehavior = NSWindowCollectionBehaviorFullScreenNone;
+                       PTYSession *session = [term createTabWithProfile:profile withCommand:command];
+                       session.isSingleUseSession = YES;
+                       return session;
+                   }];
 }
 
 @end
