@@ -10,6 +10,7 @@
 #import "ITAddressBookMgr.h"
 #import "iTermCursor.h"
 #import "NSColor+iTerm.h"
+#import "NSDictionary+iTerm.h"
 #import "PreferencePanel.h"
 
 #define PROFILE_BLOCK(x) [[^id(Profile *profile) { return [self x:profile]; } copy] autorelease]
@@ -48,6 +49,17 @@ NSString *const kProfilePreferenceInitialDirectoryAdvancedValue = @"Advanced";
     [self setObject:@(value) forKey:key inProfile:profile model:model];
 }
 
++ (NSInteger)integerForKey:(NSString *)key inProfile:(Profile *)profile {
+    return [[self objectForKey:key inProfile:profile] integerValue];
+}
+
++ (void)setInteger:(NSInteger)value
+            forKey:(NSString *)key
+         inProfile:(Profile *)profile
+             model:(ProfileModel *)model {
+    [self setObject:@(value) forKey:key inProfile:profile model:model];
+}
+
 + (NSUInteger)unsignedIntegerForKey:(NSString *)key inProfile:(Profile *)profile {
     return [[self objectForKey:key inProfile:profile] unsignedIntegerValue];
 }
@@ -132,6 +144,55 @@ NSString *const kProfilePreferenceInitialDirectoryAdvancedValue = @"Advanced";
 
 #pragma mark - Private
 
++ (BOOL)valueIsLegal:(id)value forKey:(NSString *)key {
+    NSArray *string = @[ KEY_NAME, KEY_BADGE_FORMAT, KEY_ANSWERBACK_STRING ];
+
+    NSArray *color = @[ KEY_FOREGROUND_COLOR, KEY_BACKGROUND_COLOR, KEY_BOLD_COLOR,
+                        KEY_LINK_COLOR, KEY_SELECTION_COLOR, KEY_SELECTED_TEXT_COLOR,
+                        KEY_CURSOR_COLOR, KEY_CURSOR_TEXT_COLOR, KEY_ANSI_0_COLOR,
+                        KEY_ANSI_1_COLOR, KEY_ANSI_2_COLOR, KEY_ANSI_3_COLOR, KEY_ANSI_4_COLOR,
+                        KEY_ANSI_5_COLOR, KEY_ANSI_6_COLOR, KEY_ANSI_7_COLOR, KEY_ANSI_8_COLOR,
+                        KEY_ANSI_9_COLOR, KEY_ANSI_10_COLOR, KEY_ANSI_11_COLOR, KEY_ANSI_12_COLOR,
+                        KEY_ANSI_13_COLOR, KEY_ANSI_14_COLOR, KEY_ANSI_15_COLOR,
+                        KEY_CURSOR_GUIDE_COLOR, KEY_BADGE_COLOR, KEY_TAB_COLOR,
+                        KEY_UNDERLINE_COLOR ];
+
+    NSArray *number = @[ KEY_USE_CURSOR_GUIDE, KEY_USE_TAB_COLOR, KEY_USE_UNDERLINE_COLOR,
+                         KEY_SMART_CURSOR_COLOR, KEY_MINIMUM_CONTRAST, KEY_CURSOR_BOOST,
+                         KEY_CURSOR_TYPE, KEY_BLINKING_CURSOR, KEY_USE_BOLD_FONT, KEY_THIN_STROKES,
+                         KEY_ASCII_LIGATURES, KEY_NON_ASCII_LIGATURES, KEY_USE_BRIGHT_BOLD,
+                         KEY_BLINK_ALLOWED, KEY_USE_ITALIC_FONT, KEY_AMBIGUOUS_DOUBLE_WIDTH,
+                         KEY_UNICODE_NORMALIZATION, KEY_HORIZONTAL_SPACING, KEY_VERTICAL_SPACING,
+                         KEY_USE_NONASCII_FONT, KEY_TRANSPARENCY, KEY_BLUR, KEY_BLUR_RADIUS,
+                         KEY_BACKGROUND_IMAGE_TILED, KEY_BLEND, KEY_SYNC_TITLE,
+                         KEY_DISABLE_WINDOW_RESIZING,
+                         KEY_TRANSPARENCY_AFFECTS_ONLY_DEFAULT_BACKGROUND_COLOR,
+                         KEY_ASCII_ANTI_ALIASED, KEY_NONASCII_ANTI_ALIASED, KEY_SCROLLBACK_LINES,
+                         KEY_UNLIMITED_SCROLLBACK, KEY_SCROLLBACK_WITH_STATUS_BAR,
+                         KEY_SCROLLBACK_IN_ALTERNATE_SCREEN, KEY_CHARACTER_ENCODING,
+                         KEY_XTERM_MOUSE_REPORTING, KEY_XTERM_MOUSE_REPORTING_ALLOW_MOUSE_WHEEL,
+                         KEY_UNICODE_VERSION, KEY_ALLOW_TITLE_REPORTING, KEY_ALLOW_TITLE_SETTING,
+                         KEY_DISABLE_PRINTING, KEY_DISABLE_SMCUP_RMCUP, KEY_SILENCE_BELL,
+                         KEY_BOOKMARK_GROWL_NOTIFICATIONS, KEY_SEND_BELL_ALERT, KEY_SEND_IDLE_ALERT,
+                         KEY_SEND_NEW_OUTPUT_ALERT, KEY_SEND_SESSION_ENDED_ALERT,
+                         KEY_SEND_TERMINAL_GENERATED_ALERT, KEY_FLASHING_BELL, KEY_VISUAL_BELL,
+                         KEY_CLOSE_SESSIONS_ON_END, KEY_PROMPT_CLOSE,
+                         KEY_UNDO_TIMEOUT, KEY_REDUCE_FLICKER, KEY_SEND_CODE_WHEN_IDLE,
+                         KEY_IDLE_CODE, KEY_IDLE_PERIOD, KEY_OPTION_KEY_SENDS,
+                         KEY_RIGHT_OPTION_KEY_SENDS, KEY_APPLICATION_KEYPAD_ALLOWED,
+                         KEY_PLACE_PROMPT_AT_FIRST_COLUMN, KEY_SHOW_MARK_INDICATORS,
+                       ];
+    if ([string containsObject:key]) {
+        return [value isKindOfClass:[NSString class]];
+    } else if ([color containsObject:key]) {
+        return [value isKindOfClass:[NSDictionary class]] && [(NSDictionary *)value isColorValue];
+    } else if ([number containsObject:key]) {
+        return [value isKindOfClass:[NSNumber class]];
+    } else {
+        return NO;
+    }
+}
+
 + (NSDictionary *)defaultValueMap {
     static NSDictionary *dict;
     if (!dict) {
@@ -174,6 +235,8 @@ NSString *const kProfilePreferenceInitialDirectoryAdvancedValue = @"Advanced";
                   KEY_USE_CURSOR_GUIDE: @NO,
                   KEY_TAB_COLOR: [NSNull null],
                   KEY_USE_TAB_COLOR: @NO,
+                  KEY_UNDERLINE_COLOR: [NSNull null],
+                  KEY_USE_UNDERLINE_COLOR: @NO,
                   KEY_SMART_CURSOR_COLOR: @NO,
                   KEY_MINIMUM_CONTRAST: @0.0,
                   KEY_CURSOR_BOOST: @0.0,
@@ -181,11 +244,14 @@ NSString *const kProfilePreferenceInitialDirectoryAdvancedValue = @"Advanced";
                   KEY_BLINKING_CURSOR: @NO,
                   KEY_USE_BOLD_FONT: @YES,
                   KEY_THIN_STROKES: @(iTermThinStrokesSettingRetinaOnly),
+                  KEY_ASCII_LIGATURES: @NO,
+                  KEY_NON_ASCII_LIGATURES: @NO,
                   KEY_USE_BRIGHT_BOLD: @YES,
                   KEY_BLINK_ALLOWED: @NO,
                   KEY_USE_ITALIC_FONT: @YES,
                   KEY_AMBIGUOUS_DOUBLE_WIDTH: @NO,
                   KEY_USE_HFS_PLUS_MAPPING: @NO,
+                  KEY_UNICODE_NORMALIZATION: @(iTermUnicodeNormalizationNone),
                   KEY_HORIZONTAL_SPACING: @1.0,
                   KEY_VERTICAL_SPACING: @1.0,
                   KEY_USE_NONASCII_FONT: @YES,
@@ -199,7 +265,7 @@ NSString *const kProfilePreferenceInitialDirectoryAdvancedValue = @"Advanced";
                   KEY_HIDE_AFTER_OPENING: @NO,
                   KEY_WINDOW_TYPE: @(WINDOW_TYPE_NORMAL),
                   KEY_SCREEN: @-1,
-                  KEY_SPACE: @0,
+                  KEY_SPACE: @(iTermProfileOpenInCurrentSpace),
                   KEY_SYNC_TITLE: @NO,
                   KEY_DISABLE_WINDOW_RESIZING: @NO,
                   KEY_PREVENT_TAB: @NO,
@@ -215,6 +281,8 @@ NSString *const kProfilePreferenceInitialDirectoryAdvancedValue = @"Advanced";
                   KEY_TERMINAL_TYPE: @"xterm",
                   KEY_ANSWERBACK_STRING: @"",
                   KEY_XTERM_MOUSE_REPORTING: @NO,
+                  KEY_XTERM_MOUSE_REPORTING_ALLOW_MOUSE_WHEEL: @YES,
+                  KEY_UNICODE_VERSION: @8,
                   KEY_ALLOW_TITLE_REPORTING: @NO,
                   KEY_ALLOW_TITLE_SETTING: @YES,
                   KEY_DISABLE_PRINTING: @NO,
@@ -244,6 +312,23 @@ NSString *const kProfilePreferenceInitialDirectoryAdvancedValue = @"Advanced";
                   KEY_APPLICATION_KEYPAD_ALLOWED: @NO,
                   KEY_PLACE_PROMPT_AT_FIRST_COLUMN: @YES,
                   KEY_SHOW_MARK_INDICATORS: @YES,
+                  KEY_HAS_HOTKEY: @NO,
+                  KEY_HOTKEY_MODIFIER_FLAGS: @0,
+                  KEY_HOTKEY_CHARACTERS: @"",
+                  KEY_HOTKEY_CHARACTERS_IGNORING_MODIFIERS: @"",
+                  KEY_HOTKEY_KEY_CODE: @0,
+                  KEY_HOTKEY_AUTOHIDE: @YES,
+                  KEY_HOTKEY_REOPEN_ON_ACTIVATION: @NO,
+                  KEY_HOTKEY_ANIMATE: @YES,
+                  KEY_HOTKEY_FLOAT: @YES,
+                  KEY_HOTKEY_DOCK_CLICK_ACTION: @(iTermHotKeyDockPreferenceDoNotShow),
+                  KEY_HOTKEY_MODIFIER_ACTIVATION: @0,
+                  KEY_HOTKEY_ACTIVATE_WITH_MODIFIER: @NO,
+                  KEY_HOTKEY_ALTERNATE_SHORTCUTS: @[],
+                  KEY_SESSION_HOTKEY: @{},
+                  // Remember to update valueIsLegal:forKey: and the websocket
+                  // README.md when adding a new value that should be
+                  // API-settable.
                 };
         [dict retain];
     }
@@ -269,6 +354,16 @@ NSString *const kProfilePreferenceInitialDirectoryAdvancedValue = @"Advanced";
                                                       userInfo:nil];
 }
 
++ (void)setObjectsFromDictionary:(NSDictionary *)dictionary
+                       inProfile:(Profile *)profile
+                           model:(ProfileModel *)model {
+    [model setObjectsFromDictionary:dictionary inProfile:profile];
+    [model flush];
+    [[NSNotificationCenter defaultCenter] postNotificationName:kReloadAllProfiles
+                                                        object:nil
+                                                      userInfo:nil];
+}
+
 + (id)defaultObjectForKey:(NSString *)key {
     id obj = [self defaultValueMap][key];
     if ([obj isKindOfClass:[NSNull class]]) {
@@ -286,7 +381,10 @@ NSString *const kProfilePreferenceInitialDirectoryAdvancedValue = @"Advanced";
 + (NSDictionary *)computedObjectDictionary {
     static NSDictionary *dict;
     if (!dict) {
-        dict = @{ KEY_IDLE_PERIOD: PROFILE_BLOCK(antiIdlePeriodWithLegacyDefaultInProfile) };
+        dict = @{ KEY_IDLE_PERIOD: PROFILE_BLOCK(antiIdlePeriodWithLegacyDefaultInProfile),
+                  KEY_UNICODE_NORMALIZATION: PROFILE_BLOCK(unicodeNormalizationForm),
+                  KEY_UNICODE_VERSION: PROFILE_BLOCK(unicodeVersion),
+                };
         [dict retain];
     }
     return dict;
@@ -328,4 +426,42 @@ NSString *const kProfilePreferenceInitialDirectoryAdvancedValue = @"Advanced";
     return [self defaultObjectForKey:key];
 }
 
++ (id)unicodeNormalizationForm:(Profile *)profile {
+    NSString *const key = KEY_UNICODE_NORMALIZATION;
+
+    // If the profile has a value.
+    NSNumber *value = profile[key];
+    if (value) {
+        return value;
+    }
+
+    // If the deprecated boolean was set, use it
+    value = profile[KEY_USE_HFS_PLUS_MAPPING];
+    if (value) {
+        return value.boolValue ? @(iTermUnicodeNormalizationHFSPlus) : @(iTermUnicodeNormalizationNone);
+    }
+
+    // Fall back to the default from the dictionary.
+    return [self defaultObjectForKey:key];
+}
+
++ (id)unicodeVersion:(Profile *)profile {
+    NSString *const key = KEY_UNICODE_VERSION;
+
+    // If the profile has a value.
+    NSNumber *value = profile[key];
+    if (value) {
+        return value;
+    }
+
+    if (@available(macOS 10.13, *)) {
+        // macOS 10.13 has switched to unicode 9 widths. If you're sshing somewhere then you're
+        // going to have a bad time. My hope is that this makes people happier on balance.
+        return @9;
+    } else {
+        // Fall back to the default from the dictionary.
+        return [self defaultObjectForKey:key];
+    }
+}
+
 @end
